@@ -2,31 +2,23 @@
 #include <nsis/pluginapi.h> // nsis plugin
 #include "../../Source/exehead/resource.h"
 
-// Turn a pair of chars into a word
-// Turn four chars into a dword
-#ifdef __BIG_ENDIAN__ // Not very likely, but, still...
-#define CHAR2_TO_WORD(a,b) (((WORD)(b))|((a)<<8))
-#define CHAR4_TO_DWORD(a,b,c,d)	(((DWORD)CHAR2_TO_WORD(c,d))|(CHAR2_TO_WORD(a,b)<<16))
-#else
-#define CHAR2_TO_WORD(a,b) (((WORD)(a))|((b)<<8))
-#define CHAR4_TO_DWORD(a,b,c,d)	(((DWORD)CHAR2_TO_WORD(a,b))|(CHAR2_TO_WORD(c,d)<<16))
-#endif
+// Unicode support added by Jim Park -- 08/02/2007
 
 HINSTANCE hInstance;
 HWND hwBanner;
 HANDLE hThread;
 BOOL bFailed;
 
-char buf[1024];
+TCHAR buf[1024];
 
-BOOL CALLBACK BannerProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
+INT_PTR CALLBACK BannerProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
 {
   if (uMsg == WM_INITDIALOG)
   {
     int iMainStringSet = 0;
 
     popstring(buf);
-    while (*(int*)buf == CHAR4_TO_DWORD('/','s','e','t') && !buf[4])
+    while (lstrcmp(buf, _T("/set")) == 0)
     {
       unsigned int id;
       popstring(buf);
@@ -44,13 +36,13 @@ BOOL CALLBACK BannerProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
       SetDlgItemText(hwndDlg, IDC_STR, buf);
 
     if (!*buf)
-      SetWindowLong(hwndDlg, GWL_EXSTYLE, GetWindowLong(hwndDlg, GWL_EXSTYLE) | WS_EX_TOOLWINDOW);
+      SetWindowLongPtr(hwndDlg, GWL_EXSTYLE, GetWindowLongPtr(hwndDlg, GWL_EXSTYLE) | WS_EX_TOOLWINDOW);
   }
   if (uMsg == WM_CLOSE)
   {
     DestroyWindow(hwndDlg);
   }
-  return 0;
+  return FALSE;
 }
 
 BOOL ProcessMessages()
@@ -116,7 +108,7 @@ static UINT_PTR PluginCallback(enum NSPIM msg)
   return 0;
 }
 
-void __declspec(dllexport) show(HWND hwndParent, int string_size, char *variables, stack_t **stacktop, extra_parameters *extra)
+void __declspec(dllexport) show(HWND hwndParent, int string_size, TCHAR *variables, stack_t **stacktop, extra_parameters *extra)
 {
   EXDLL_INIT();
 
@@ -155,13 +147,13 @@ void __declspec(dllexport) show(HWND hwndParent, int string_size, char *variable
   }
 }
 
-void __declspec(dllexport) getWindow(HWND hwndParent, int string_size, char *variables, stack_t **stacktop)
+void __declspec(dllexport) getWindow(HWND hwndParent, int string_size, TCHAR *variables, stack_t **stacktop)
 {
-  wsprintf(buf, "%u", hwBanner);
+  wsprintf(buf, _T("%u"), hwBanner);
   pushstring(buf);
 }
 
-void __declspec(dllexport) destroy(HWND hwndParent, int string_size, char *variables, stack_t **stacktop)
+void __declspec(dllexport) destroy(HWND hwndParent, int string_size, TCHAR *variables, stack_t **stacktop)
 {
   if (!hwBanner)
     return;
