@@ -37,6 +37,11 @@
 #define SupportsWNT4() ( sizeof(void*) == 4 && !DpiAwarePerMonitor() ) // NT4 does not support the MultiMon API
 #define SupportsW9X() ( sizeof(TCHAR) == 1 )
 #define SupportsW95() ( FALSE && SupportsW9X() && !DpiAwarePerMonitor() )
+#define SupportsW2000() ( sizeof(void*) == 4 )
+#define SupportsRTLUI() ( FALSE ) // UI is English only
+
+static inline bool IsWin9598ME() { return SupportsW9X() && (int) GetVersion() < 0; }
+static inline bool IsWin95() { return SupportsW95() && (GetVersion() & (0x8000FFFF & ~0x0300)) == 0x80000004; }
 
 // Defines
 #define NSIS_URL     "https://nsis.sourceforge.io/"
@@ -50,7 +55,7 @@
 #endif
 #define COPYRIGHT    _T("Copyright (C) 2002 Robert Rainwater")
 #define CONTRIB      _T("Fritz Elfert, Justin Frankel, Amir Szekely, Sunil Kamath, Joost Verburg, Anders Kjersem")
-#define DONATE       _T("<A ID=\"D\">Donate</A> to support NSIS development.")
+#define DONATE       _T("<A>Donate</A> to support NSIS development.")
 #define DONATEURL    { 'h'|x,'t','t'|x,'p',':'|x,'/'|x,'/'|x,'n'|x,'s'|x,'i'|x,'s'|x,'.'|x,'s'|x,'f'|x,'.','n'|x,'e'|x,'t'|x,'/'|x,'r','/'|x,'D'|x,'o'|x,'n'|x,'a'|x,'t'|x,'e'|x,'\0' } // "Encrypted"
 #define DOCPATH      "https://nsis.sourceforge.io/Docs/"
 #define LOCALDOCS    _T("\\NSIS.chm")
@@ -84,11 +89,15 @@
 #define SAVE_BUTTON_TEXT _T("Save")
 #define LOAD_SYMBOL_SET_MESSAGE _T("Please select a name for the Symbol Definitions Set to load.")
 #define SAVE_SYMBOL_SET_MESSAGE _T("Please enter or select a name for the Symbol Definitions Set to save.")
+#define TESTBTN_TIPTEXT _T("Test the generated installer")
+#define CLOSEBTN_TIPTEXT _T("Close MakeNSISW")
 
 #define WM_MAKENSIS_PROCESSCOMPLETE (WM_USER+1001)
 #define WM_MAKENSIS_LOADSYMBOLSET (WM_USER+1002)
 #define WM_MAKENSIS_SAVESYMBOLSET (WM_USER+1003)
 #define WM_MAKENSIS_UPDATEUISTATE (WM_USER+1004)
+#define WM_MAKENSIS_FREEZEEDITOR (WM_USER+1005)
+#define TID_CONFIGURECLOSEORABORT 1
 
 namespace MakensisAPI {
   extern const TCHAR* SigintEventNameFmt;
@@ -189,12 +198,14 @@ typedef struct {
   UINT AnimSpeed, FinalHeaderPos;
 } ABOUTDLGDATA;
 INT_PTR ShowAboutDialog(HWND hwndOwner);
+
 INT_PTR CALLBACK SettingsProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam);
 typedef struct {
   void*pOldMDD;
   BOOL LoadingMode;
 } SYMSETDLGDATA;
 INT_PTR ShowSymbolSetDialog(HWND hwndOwner, BOOL LoadingSet);
+
 INT_PTR CALLBACK CompressorProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam);
 void           SetScript(const TCHAR *script, bool clearArgs = true);
 void           CompileNSISScript();
@@ -219,12 +230,12 @@ typedef struct NSISScriptData {
   char  *brandingv;
   TCHAR **symbols;
   int retcode;
-  bool userSelectCompressor;
   unsigned char verbosity;
   DWORD logLength;
   DWORD warnings;
   HINSTANCE hInstance;
   HWND hwnd;
+  HWND logwnd;
   HMENU menu;
   HMENU fileSubmenu;
   HMENU editSubmenu;
@@ -233,13 +244,14 @@ typedef struct NSISScriptData {
   HANDLE sigint_event;
   HANDLE sigint_event_legacy;
   HWND focused_hwnd;
+  void*pLogTextDoc;
   NCOMPRESSOR default_compressor;
   NCOMPRESSOR compressor;
   LPCTSTR compressor_name;
   TCHAR compressor_stats[512];
   LPCTSTR best_compressor_name;
-  // Added by Darren Owen (DrO) on 1/10/2003
   int recompile_test;
+  WORD log_zoom;
 } NSCRIPTDATA;
 
 extern NSCRIPTDATA g_sdata;
@@ -264,4 +276,8 @@ typedef struct ToolTipStruct {
   HHOOK hook;
 } NTOOLTIP;
 
+
+extern INT_PTR ShowWndSpy(HWND hOwner);
+extern INT_PTR ShowLookupDialog(HWND hOwner);
+
 #endif
