@@ -60,6 +60,9 @@ HMODULE LoadSysLibrary(LPCSTR Mod)
   return LoadLibrary(path);
 }
 
+#ifdef DECLSPEC_NOINLINE
+DECLSPEC_NOINLINE
+#endif
 FARPROC GetSysProcAddr(LPCSTR Mod, LPCSTR FuncName)
 {
   return GetProcAddress(LoadSysLibrary(Mod), FuncName);
@@ -76,6 +79,20 @@ bool WriteUTF16LEBOM(HANDLE hFile)
   return WriteFile(hFile, u16lb, sizeof(u16lb));
 }
 
+BOOL InitCCExHelper(UINT icc) {
+  INITCOMMONCONTROLSEX icx = { sizeof(icx), icc };
+  BOOL suppw95 = SupportsW95();
+  FARPROC icce = suppw95 ? GetSysProcAddr("COMCTL32", "InitCommonControlsEx") : (FARPROC) InitCommonControlsEx;
+  return (!suppw95 || icce) && ((BOOL(WINAPI*)(const INITCOMMONCONTROLSEX*))icce)(&icx);
+}
+
+UINT GetScreenBPP(HWND hWnd) {
+  HDC hDc = GetDC(hWnd);
+  UINT bpp = GetDeviceCaps(hDc, BITSPIXEL) * GetDeviceCaps(hDc, PLANES); // TODO: COLORRES if RASTERCAPS&RC_PALETTE?
+  ReleaseDC(hWnd, hDc);
+  return bpp;
+}
+
 int SetArgv(const TCHAR *cmdLine, TCHAR ***argv) {
   const TCHAR *p;
   TCHAR *arg, *argSpace;
@@ -161,8 +178,7 @@ int SetArgv(const TCHAR *cmdLine, TCHAR ***argv) {
 
 void SetTitle(HWND hwnd,const TCHAR *substr) {
   TCHAR title[64];
-  if (substr==NULL) wsprintf(title,_T("MakeNSISW"));
-  else wsprintf(title,_T("MakeNSISW - %s"),substr); 
+  wsprintf(title,substr ? _T("MakeNSISW - %s") : _T("MakeNSISW"),substr);
   SetWindowText(hwnd,title);
 }
 
@@ -180,21 +196,21 @@ void PlayAppSoundAsync(LPCSTR SoundName, int MBFallback) {
   PLAYAPPSOUNDDATA *p = (PLAYAPPSOUNDDATA*) MemAlloc(sizeof(PLAYAPPSOUNDDATA));
   if (p) {
     p->SoundName = SoundName, p->MBFallback = MBFallback; // Note: The string must be valid until the sound has started because we don't copy it
-    HANDLE hThread = CreateThread(NULL, 0, PlayAppSoundProc, p, 0, &tid);
+    HANDLE hThread = CreateThread(NULL, 0, PlayAppSoundProc, p, 0, SupportsW9X() ? &tid : 0);
     if (hThread) CloseHandle(hThread); else PlayAppSoundProc(p);
   }
 }
 
 void CopyToClipboard(HWND hwnd) {
   if (!hwnd || !OpenClipboard(hwnd)) return;
-  LRESULT len = SendDlgItemMessage(hwnd, IDC_LOGWIN, WM_GETTEXTLENGTH, 0, 0);
+  LRESULT len = SendMessage(g_sdata.logwnd, WM_GETTEXTLENGTH, 0, 0);
   HGLOBAL mem = GlobalAlloc(GMEM_MOVEABLE, (++len)*sizeof(TCHAR));
   if (!mem) { CloseClipboard(); return; }
   TCHAR *txt = (TCHAR *)GlobalLock(mem);
   if (!txt) { CloseClipboard(); return; }
   EmptyClipboard();
   txt[0] = 0;
-  SendDlgItemMessage(hwnd, IDC_LOGWIN, WM_GETTEXT, (WPARAM)(len), (LPARAM)txt);
+  SendMessage(g_sdata.logwnd, WM_GETTEXT, (WPARAM)(len), (LPARAM)txt);
   GlobalUnlock(mem);
 #ifdef _UNICODE
   SetClipboardData(CF_UNICODETEXT, mem);
@@ -204,10 +220,50 @@ void CopyToClipboard(HWND hwnd) {
   CloseClipboard();
 }
 
+#include <shlobj.h>
+#if defined(MSFTEDIT_CLASS)
+#include <tom.h>
+#define RE_HAS_TOM
+#define IID_ITextDocument NSIS_IID_ITextDocument
+static const GUID IID_ITextDocument = { 0x8cc497c0, 0xa1df, 0x11ce, { 0x80,0x98,0x0,0xaa,0x0,0x47,0xbe,0x5d } };
+#endif
+void ReleaseLogWindow() {
+#ifdef RE_HAS_TOM
+  if (g_sdata.pLogTextDoc) ((IUnknown*)g_sdata.pLogTextDoc)->Release();
+#endif
+}
+void InitializeLogWindow() {
+  HWND hRE = g_sdata.logwnd;
+#ifdef RE_HAS_TOM
+  IUnknown *pTD = 0, *pREO;
+  if (SendMessage(hRE, EM_GETOLEINTERFACE, 0, (LPARAM)&pREO) && pREO) {
+    if (FAILED(pREO->QueryInterface(IID_ITextDocument, (void**) &pTD))) pTD = 0;
+    pREO->Release();
+  }
+  g_sdata.pLogTextDoc = pTD;
+#endif
+  SendMessage(hRE, EM_SETTEXTMODE, TM_PLAINTEXT, 0);
+}
+
+HRESULT RicheditFreeze(void*pITextDocument, SIZE_T Freeze)
+{
+  HRESULT hr = E_NOTIMPL;
+#ifdef RE_HAS_TOM
+  ITextDocument*pTD = (ITextDocument*) pITextDocument;
+  if (pTD) {
+    if (Freeze)
+      hr = pTD->Freeze(0);
+    else
+      hr = pTD->Unfreeze(0);
+  }
+#endif
+  return hr;
+}
+
 void SetLogColor(enum LOGCOLOR lc)
 {
   enum { em_seteditstyle = (WM_USER + 204), ses_extendbackcolor = 4 };
-  HWND hEd = GetDlgItem(g_sdata.hwnd, IDC_LOGWIN);
+  HWND hEd = g_sdata.logwnd;
   bool sysclr = lc >= LC_SYSCOLOR || !ReadRegSettingDW(REGCOLORIZE, true);
   static const COLORREF clrs[] = { RGB(0, 50, 0), RGB(210, 255, 210), RGB(50, 30, 0), RGB(255, 220, 190), RGB(50, 0, 0), RGB(255, 210, 210) };
   CHARFORMAT cf;
@@ -219,17 +275,24 @@ void SetLogColor(enum LOGCOLOR lc)
   SendMessage(hEd, EM_SETBKGNDCOLOR, sysclr, sysclr ? sysclr /*Irrelevant*/ : clrs[(lc * 2) + 1]);
 }
 
-void ClearLog(HWND hwnd) {
-  SetDlgItemText(hwnd, IDC_LOGWIN, _T(""));
+void ClearLog() {
+  SetWindowText(g_sdata.logwnd, _T(""));
   SetLogColor(LC_SYSCOLOR);
   SendMessage(g_sdata.hwnd, WM_MAKENSIS_UPDATEUISTATE, 0, 0);
 }
 
 void LogMessage(HWND hwnd,const TCHAR *str) {
-  SendDlgItemMessage(hwnd, IDC_LOGWIN, EM_SETSEL, g_sdata.logLength, g_sdata.logLength);
+  HWND hLogWin = g_sdata.logwnd;
+#ifdef RE_HAS_TOM
+  HRESULT hr = (HRESULT) SendMessage(hwnd, WM_MAKENSIS_FREEZEEDITOR, 0, true); // Force COM calls to UI thread
+#endif
+  SendMessage(hLogWin, EM_SETSEL, g_sdata.logLength, g_sdata.logLength);
+  SendMessage(hLogWin, EM_REPLACESEL, 0, (LPARAM)str);
+  SendMessage(hLogWin, EM_SCROLLCARET, 0, 0);
+#ifdef RE_HAS_TOM
+  if (SUCCEEDED(hr)) SendMessage(hwnd, WM_MAKENSIS_FREEZEEDITOR, 0, false);
+#endif
   g_sdata.logLength += lstrlen(str);
-  SendDlgItemMessage(hwnd, IDC_LOGWIN, EM_REPLACESEL, 0, (LPARAM)str);
-  SendDlgItemMessage(hwnd, IDC_LOGWIN, EM_SCROLLCARET, 0, 0);
 }
 
 void ErrorMessage(HWND hwnd,const TCHAR *str) {
@@ -274,10 +337,11 @@ HWND GetComboEdit(HWND hCB)
   return FindWindowEx(hCB, hList, 0, 0);
 }
 
-void EnableDisableItems(HWND hwnd, int on) 
+void EnableDisableItems(int on) 
 {
-  const HWND hCloseBtn = GetDlgItem(hwnd, IDCANCEL);
-  const HWND hTestBtn = GetDlgItem(hwnd, IDC_TEST);
+  HWND hwndDlg = g_sdata.hwnd;
+  const HWND hCloseBtn = GetDlgItem(hwndDlg, IDCANCEL);
+  const HWND hTestBtn = GetDlgItem(hwndDlg, IDC_TEST);
   const HMENU hMenu = g_sdata.menu;
   const UINT mf = (!on ? MF_GRAYED : MF_ENABLED);
   const UINT nmf = (!on ? MF_ENABLED : MF_GRAYED);
@@ -291,7 +355,7 @@ void EnableDisableItems(HWND hwnd, int on)
     EnableMenuItem(hMenu, IDM_TEST, mf);
   }
   EnableMenuItem(hMenu, IDM_CANCEL, nmf);
-  EnableWindow(hCloseBtn, on);
+  EnableWindow(hCloseBtn, false);
 
   static const PACKEDCMDID_T cmds [] = {
     PACKCMDID(IDM_EXIT), PACKCMDID(IDM_LOADSCRIPT), PACKCMDID(IDM_EDITSCRIPT), 
@@ -306,15 +370,16 @@ void EnableDisableItems(HWND hwnd, int on)
       EnableToolBarButton(id, on);
   }
 
-  SendMessage(g_sdata.hwnd, WM_MAKENSIS_UPDATEUISTATE, 0 ,0);
+  SendMessage(hwndDlg, WM_MAKENSIS_UPDATEUISTATE, 0 ,0);
   EnableMenuItem(hMenu, IDM_FILE, mf); // Disable the whole File menu because of the MRU list
-  DrawMenuBar(g_sdata.hwnd);
+  DrawMenuBar(hwndDlg);
 
   HWND hFocus = g_sdata.focused_hwnd, hOptimal = hTestBtn;
   if (on && hCloseBtn == hFocus) hFocus = hOptimal;
-  if (!IsWindowEnabled(hFocus)) hFocus = GetDlgItem(hwnd, IDC_LOGWIN);
-  SetDialogFocus(hwnd, hOptimal);
-  SetDialogFocus(hwnd, hFocus);
+  if (!IsWindowEnabled(hFocus)) hFocus = g_sdata.logwnd;
+  SetDialogFocus(hwndDlg, hOptimal);
+  SetDialogFocus(hwndDlg, hFocus);
+  SetTimer(hwndDlg, TID_CONFIGURECLOSEORABORT, 1000, 0);
 }
 
 void SetCompressorStats()
@@ -323,10 +388,10 @@ void SetCompressorStats()
   TCHAR buf[1024];
   bool found = false;
 
-  line_count = SendDlgItemMessage(g_sdata.hwnd, IDC_LOGWIN, EM_GETLINECOUNT, 0, 0);
+  line_count = SendMessage(g_sdata.logwnd, EM_GETLINECOUNT, 0, 0);
   for(i=0; i<line_count; i++) {
     *((LPWORD)buf) = COUNTOF(buf); 
-    LRESULT cchLine = SendDlgItemMessage(g_sdata.hwnd, IDC_LOGWIN, EM_GETLINE, (WPARAM)i, (LPARAM)buf);
+    LRESULT cchLine = SendMessage(g_sdata.logwnd, EM_GETLINE, (WPARAM)i, (LPARAM)buf);
     buf[cchLine] = _T('\0');
     if(found) {
       DWORD len = lstrlen(TOTAL_SIZE_COMPRESSOR_STAT);
@@ -359,7 +424,7 @@ static void SetUIState_NoScript()
 
 void CompileNSISScript() {
   DragAcceptFiles(g_sdata.hwnd,FALSE);
-  ClearLog(g_sdata.hwnd);
+  ClearLog();
   SetTitle(g_sdata.hwnd,NULL);
   PostMessage(g_sdata.hwnd, WM_MAKENSIS_UPDATEUISTATE, 0, 0);
   if (!g_sdata.script[0]) {
@@ -436,7 +501,7 @@ static DWORD RegReadString(HKEY hKey, LPCTSTR Name, LPTSTR Buf, DWORD cbBufSize)
   return ec;
 }
 
-static DWORD RegOpenKeyForReading(HKEY hRoot, LPCTSTR SubKey, HKEY*pKey) {
+DWORD RegOpenKeyForReading(HKEY hRoot, LPCTSTR SubKey, HKEY*pKey) {
   return RegOpenKeyEx(hRoot, SubKey, 0, KEY_READ, pKey);
 }
 
@@ -519,9 +584,9 @@ void SaveWindowPos(HWND hwnd) {
   HKEY hKey;
   WINDOWPLACEMENT p;
   p.length = sizeof(p);
-  GetWindowPlacement(hwnd, &p);
-  if (CreateRegSettingsKey(hKey)) {
-    RegSetValueEx(hKey, REGLOC, 0, REG_BINARY, (LPBYTE)&p, sizeof(p));
+  if (!GetWindowPlacement(hwnd, &p)) p.length = 0;
+  if (p.length && CreateRegSettingsKey(hKey)) {
+    RegSetValueEx(hKey, REGLOC, 0, REG_BINARY, (LPBYTE)&p, p.length);
     RegCloseKey(hKey);
   }
 }
@@ -701,21 +766,31 @@ int InitBranding() {
   return retval;
 }
 
+LRESULT SetTooltipText(HWND hWnd, UINT_PTR Id, LPCTSTR Text) {
+  TOOLINFO ti;
+  ti.cbSize = SizeOfStruct(ti);
+  ti.uFlags = Id > 0x7fff ? TTF_IDISHWND : 0;
+  ti.hwnd = hWnd, ti.uId = Id;
+  ti.hinst = g_sdata.hInstance, ti.lpszText = const_cast<LPTSTR>(Text);
+  return SendMessage(g_tip.tip, TTM_UPDATETIPTEXT, 0, (LPARAM) (LPTOOLINFO) &ti); 
+}
+
+void UpdateCloseButtonTooltip() {
+  LPCTSTR txt = g_sdata.thread ? _T("") : CLOSEBTN_TIPTEXT;
+  SetTooltipText(g_tip.tip_p, (UINT_PTR) GetDlgItem(g_sdata.hwnd, IDCANCEL), txt);
+}
+
 void InitTooltips(HWND h) {
   if (h == NULL)  return;
-  memset(&g_tip,0,sizeof(NTOOLTIP));
+  InitCCEx(ICC_BAR_CLASSES);
+  DWORD s = WS_POPUP | WS_BORDER | TTS_ALWAYSTIP, xs = WS_EX_TOOLWINDOW | WS_EX_TOPMOST;
+  g_tip.hook = 0;
   g_tip.tip_p = h;
-  INITCOMMONCONTROLSEX icx;
-  icx.dwSize = sizeof(icx);
-  icx.dwICC  = ICC_BAR_CLASSES;
-  InitCommonControlsEx(&icx);
-  DWORD dwStyle = WS_POPUP | WS_BORDER | TTS_ALWAYSTIP;
-  DWORD dwExStyle = WS_EX_TOOLWINDOW | WS_EX_TOPMOST;
-  g_tip.tip = CreateWindowEx(dwExStyle,TOOLTIPS_CLASS,NULL,dwStyle,0,0,0,0,h,NULL,GetModuleHandle(NULL),NULL);
+  g_tip.tip = CreateWindowEx(xs, TOOLTIPS_CLASS, NULL, s, 0, 0, 0, 0, h, NULL, HINST_APPLICATION, NULL);
   if (!g_tip.tip) return;
-  g_tip.hook = SetWindowsHookEx(WH_GETMESSAGE,TipHookProc,NULL, GetCurrentThreadId());
-  AddTip(GetDlgItem(h,IDCANCEL),_T("Close MakeNSISW"));
-  AddTip(GetDlgItem(h,IDC_TEST),_T("Test the installer generated by MakeNSISW"));
+  g_tip.hook = SetWindowsHookEx(WH_GETMESSAGE, TipHookProc, NULL, GetCurrentThreadId());
+  AddTip(GetDlgItem(h, IDCANCEL), CLOSEBTN_TIPTEXT);
+  AddTip(GetDlgItem(h, IDC_TEST), TESTBTN_TIPTEXT);
   AddToolBarTooltips();
 }
 
@@ -723,9 +798,9 @@ void DestroyTooltips() {
   UnhookWindowsHookEx(g_tip.hook);
 }
 
-void AddTip(HWND hWnd,LPCTSTR lpszToolTip) {
+void AddTip(HWND hWnd, LPCTSTR lpszToolTip) {
   TOOLINFO ti;
-  ti.cbSize = sizeof(TOOLINFO);
+  ti.cbSize = SizeOfStruct(ti);
   ti.uFlags = TTF_IDISHWND;
   ti.hwnd   = g_tip.tip_p;
   ti.uId = (UINT_PTR) hWnd;
@@ -1118,6 +1193,20 @@ HFONT CreateFontHelper(INT_PTR Data, int Height, DWORD p1, LPCTSTR Face)
   return CreateFont(Height, 0, 0, 0, w, FALSE, FALSE, FALSE, cs, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, paf, Face);
 }
 
+BOOL CALLBACK FontExistsCallback(const LOGFONT*pLF, const TEXTMETRIC*pTM, DWORD Type, LPARAM Cookie)
+{
+  *((BOOL*) Cookie) = TRUE;
+  return FALSE;
+}
+BOOL FontExists(LPCTSTR Face)
+{
+  BOOL ret = FALSE;
+  HDC hDC = GetDC(0);
+  EnumFonts(hDC, Face, FontExistsCallback, (LPARAM) &ret);
+  ReleaseDC(0, hDC);
+  return ret;
+}
+
 BOOL FillRectColor(HDC hDC, const RECT &Rect, COLORREF Color)
 {
   COLORREF orgclr = SetBkColor(hDC, Color);
