@@ -32,6 +32,14 @@
 #endif
 #define COUNTOF(a) (sizeof(a)/sizeof(a[0]))
 
+#if defined(_MSC_VER) && _MSC_VER >= 1200
+EXTERN_C IMAGE_DOS_HEADER __ImageBase;
+#define HINST_THISCOMPONENT ( (HINSTANCE) &__ImageBase )
+#define HINST_APPLICATION HINST_THISCOMPONENT
+#else
+#define HINST_APPLICATION ( (HINSTANCE) GetModuleHandle(NULL) )
+#endif
+
 #define MRU_LIST_SIZE 5
 #define MRU_DISPLAY_LENGTH 40
 #define SYMSETNAME_MAXLEN 40
@@ -46,6 +54,12 @@ HMODULE LoadSysLibrary(LPCSTR Mod);
 FARPROC GetSysProcAddr(LPCSTR Mod, LPCSTR FuncName);
 bool WriteUTF16LEBOM(HANDLE hFile);
 
+BOOL InitCCExHelper(UINT icc);
+static inline BOOL InitCCEx(UINT icc) { return icc > 0xff ? InitCCExHelper(icc) : true; }
+static inline UINT SizeOfStruct(const TOOLINFO&x) { return FIELD_OFFSET(TOOLINFO, lParam); }
+static inline UINT SizeOfStruct(const OPENFILENAME&x) { return sizeof(void*) < 8 ? 76 : sizeof(x); }
+UINT GetScreenBPP(HWND hWnd = NULL);
+
 void FreeSpawn(PROCESS_INFORMATION *pPI, HANDLE hRd, HANDLE hWr);
 BOOL InitSpawn(STARTUPINFO &si, HANDLE &hRd, HANDLE &hWr);
 
@@ -57,18 +71,20 @@ int SetArgv(const TCHAR *cmdLine, TCHAR ***argv);
 void SetTitle(HWND hwnd,const TCHAR *substr);
 void PlayAppSoundAsync(LPCSTR SoundName, int MBFallback = -1);
 void CopyToClipboard(HWND hwnd);
+void InitializeLogWindow();
+void ReleaseLogWindow();
 enum LOGCOLOR { LC_SUCCESS, LC_WARNING, LC_ERROR, LC_SYSCOLOR };
 void SetLogColor(enum LOGCOLOR lc);
-void ClearLog(HWND hwnd);
+void ClearLog();
 void LogMessage(HWND hwnd,const TCHAR *str);
 void ErrorMessage(HWND hwnd,const TCHAR *str);
 void CenterOnParent(HWND hwnd);
 void SetDialogFocus(HWND hDlg, HWND hCtl); // Use this and not SetFocus()!
 #define DlgRet(hDlg, val) ( SetWindowLongPtr((hDlg), DWLP_MSGRESULT, (val)) | TRUE )
 HWND GetComboEdit(HWND hCB);
-#define DisableItems(hwnd) EnableDisableItems(hwnd, 0)
-#define EnableItems(hwnd) EnableDisableItems(hwnd, 1)
-void EnableDisableItems(HWND hwnd, int on);
+#define DisableItems(hwnd) EnableDisableItems(((hwnd), 0))
+#define EnableItems(hwnd) EnableDisableItems(((hwnd), 1))
+void EnableDisableItems(int on);
 bool OpenRegSettingsKey(HKEY &hKey, bool create = false);
 #define CreateRegSettingsKey(refhkey) OpenRegSettingsKey((refhkey), true)
 DWORD ReadRegSettingDW(LPCTSTR name, const DWORD defval);
@@ -79,7 +95,9 @@ void ResetSymbols();
 int InitBranding();
 void InitTooltips(HWND h);
 void DestroyTooltips();
-void AddTip(HWND hWnd,LPCTSTR lpszToolTip);
+void AddTip(HWND hWnd, LPCTSTR lpszToolTip);
+LRESULT SetTooltipText(HWND hWnd, UINT_PTR Id, LPCTSTR Text);
+void UpdateCloseButtonTooltip();
 void ShowDocs();
 void RestoreCompressor();
 void SaveCompressor();
@@ -103,10 +121,34 @@ struct FSPath {
   }
 };
 
+template<class T> UINT StrLenT(const T*s) { return sizeof(T) > 1 ? lstrlenW((WCHAR*) s) : lstrlenA((CHAR*) s) ; }
+template<class T> T AsciiLoCh(T ch) { return ch >= 'A' && ch <= 'Z' ? (ch | 32) : ch; }
+
+template<class A, class B> int AsciiCmpNI(const A*a, const B*b, SIZE_T len)
+{
+	int cmp = 0;
+	for (SIZE_T i = 0;; ++i)
+		if (i == len || (cmp = AsciiLoCh(a[i]) - AsciiLoCh(b[i])) || !a[i])
+			return cmp;
+}
+template<class A, class B> inline bool AsciiMatchNI(const A*a, const B*b, SIZE_T c) { return !AsciiCmpNI(a, b, c); }
+template<class A, class B> bool AsciiMatchPrefixI(const A*s, const B*p) { return AsciiMatchNI(s, p, StrLenT(p)); }
+
+template<class T> HKEY GetRegRootKey(const T*Str)
+{
+	if (AsciiMatchPrefixI(Str, "HKEY_CLASSES_ROOT") || AsciiMatchPrefixI(Str, "HKCR")) return HKEY_CLASSES_ROOT;
+	if (AsciiMatchPrefixI(Str, "HKEY_CURRENT_USER") || AsciiMatchPrefixI(Str, "HKCU")) return HKEY_CURRENT_USER;
+	if (AsciiMatchPrefixI(Str, "HKEY_LOCAL_MACHINE") || AsciiMatchPrefixI(Str, "HKLM")) return HKEY_LOCAL_MACHINE;
+	if (AsciiMatchPrefixI(Str, "HKEY_USERS") || AsciiMatchPrefixI(Str, "HKU")) return HKEY_USERS;
+	return (HKEY) NULL;
+}
+DWORD RegOpenKeyForReading(HKEY hRoot, LPCTSTR SubKey, HKEY*pKey);
+
 bool FileExists(const TCHAR *fname);
 bool OpenUrlInDefaultBrowser(HWND hwnd, LPCSTR Url);
 
 HMENU FindSubMenu(HMENU hMenu, UINT uId);
+static inline UINT GetMenuDropAlignment() { return GetSystemMetrics(SM_MENUDROPALIGNMENT) ? TPM_RIGHTALIGN : TPM_LEFTALIGN; }
 
 typedef enum { CFF_RAWSIZE = 0x00, CFF_DPIPT = 0x01, CFF_DPIFROMHWND = 0x02 } CREATEFONTFLAGS;
 HFONT CreateFontHelper(INT_PTR Data, int Height, DWORD p1, LPCTSTR Face);
@@ -119,6 +161,7 @@ inline HFONT CreateFontPt(HWND hWndDPI, int Height, WORD Weight, BYTE PitchAndFa
 {
   return CreateFont((INT_PTR) hWndDPI, CFF_DPIFROMHWND|CFF_DPIPT, Height, Weight, PitchAndFamily, CharSet, Face);
 }
+BOOL FontExists(LPCTSTR Face);
 BOOL FillRectColor(HDC hDC, const RECT &Rect, COLORREF Color);
 BOOL DrawHorzGradient(HDC hDC, LONG l, LONG t, LONG r, LONG b, COLORREF c1, COLORREF c2);
 inline long RectW(const RECT&r) { return r.right - r.left; }
@@ -138,6 +181,7 @@ static inline void GetGripperPos(HWND hwnd, RECT&r)
 }
 
 bool RicheditHasSelection(HWND hRE);
+HRESULT RicheditFreeze(void*pITextDocument, SIZE_T Freeze);
 
 void EnableUICommand(UINT Id, INT_PTR Enabled);
 
