@@ -50,12 +50,16 @@
 **      make the socket close after sending all the data sent. 
 **  
 **   8. delete ye' ol' object.
+**
+** Unicode support by Jim Park -- 08/24/2007
+**   Keep all the stuff here strictly ANSI.
 */
 
 #ifndef _CONNECTION_H_
 #define _CONNECTION_H_
 
 #include "asyncdns.h"
+#include "netinc.h"
 
 #define JNL_CONNECTION_AUTODNS ((JNL_AsyncDNS*)-1)
 
@@ -77,7 +81,7 @@ class JNL_Connection
     ~JNL_Connection();
 
     void connect(char *hostname, int port);
-    void connect(int sock, struct sockaddr_in *loc=NULL); // used by the listen object, usually not needed by users.
+    void connect(PORTABLE_SOCKET sock, struct sockaddr_in *loc=NULL); // used by the listen object, usually not needed by users.
 
     void run(int max_send_bytes=-1, int max_recv_bytes=-1, int *bytes_sent=NULL, int *bytes_rcvd=NULL);
     int  get_state() { return m_state; }
@@ -107,7 +111,7 @@ class JNL_Connection
     short get_remote_port(void) { return m_remote_port; } // this returns the remote port of connection
   
   protected:
-    int  m_socket;
+    PORTABLE_SOCKET m_socket;
     short m_remote_port;
     char *m_recv_buffer;
     char *m_send_buffer;
