@@ -32,6 +32,16 @@
 #include <assert.h>
 #include "halibut.h"
 
+enum outputtype { OT_XHTML = 0x01, OT_CHM = 0x02, OT_HTML4 = 0x04, OT_HTML5 = 0x08 };
+
+int g_outputtype = OT_XHTML;
+#define is_chm() ( (g_outputtype & OT_CHM) == OT_CHM )
+#define is_xhtml() ( (g_outputtype & OT_XHTML) == OT_XHTML )
+#define is_html5() ( (g_outputtype & OT_HTML5) == OT_HTML5 )
+
+#define gettagtxt_br() ( is_xhtml() ? "<br />" : "<br>" )
+#define gettagtxt_hr() ( is_xhtml() ? "<hr />" : "<hr>" )
+
 struct xhtmlsection_Struct {
   struct xhtmlsection_Struct *next;     /* next sibling (NULL if split across files) */
   struct xhtmlsection_Struct *child;    /* NULL if split across files */
@@ -74,13 +84,16 @@ typedef struct {
   int leaf_smallest_contents;
   int include_version_id;
   wchar_t *author, *description;
-  wchar_t *head_end, *body, *body_start, *body_end, *address_start,
-      *address_end, *nav_attrs;
+  wchar_t *html_lang, *meta_charset;
+  wchar_t *head_start, *head_middle, *head_end, *body, *body_start, *body_end;
+  ustr_slist *meta_append;
+  wchar_t *address_start, *address_end, *nav_attrs;
   wchar_t *rlink_prefix, *rlink_suffix;
   wchar_t *chm_toc_file, *chm_ind_file;
   int suppress_address;
   xhtmlheadfmt fchapter, *fsect;
   int nfsect;
+  int keywordfragments;
 } xhtmlconfig;
 
 /*static void xhtml_level(paragraph *, int);
@@ -122,6 +135,23 @@ static xhtmlsection *currentsection;
 static FILE* chm_toc = NULL;
 static FILE* chm_ind = NULL;
 
+static const wchar_t* normalizehtmlkeywordprefix(const wchar_t*s) // [x]html... --> html...
+{
+  return s && utolower(s[0]) == 'x' && utolower(s[1]) == 'h' && utolower(s[2]) == 't' ? s + 1 : s;
+}
+
+static const wchar_t* ishtmlkeyword(const wchar_t*a, const wchar_t*b)
+{
+  return !ustricmp(normalizehtmlkeywordprefix(a), normalizehtmlkeywordprefix(b)) ? a : 0;
+}
+
+static wchar_t* configurekeyword(wchar_t**dst, const wchar_t*keyword, const paragraph*source)
+{
+  if (ishtmlkeyword(source->keyword, keyword))
+    return *dst = uadv(source->keyword);
+  else
+    return 0;
+}
 
 static xhtmlconfig xhtml_configure(paragraph * source)
 {
@@ -142,10 +172,13 @@ static xhtmlconfig xhtml_configure(paragraph * source)
   ret.include_version_id = TRUE;
   ret.author = NULL;
   ret.description = NULL;
-  ret.head_end = NULL;
+  ret.html_lang = NULL;
+  ret.meta_charset = NULL;
+  ret.head_start = ret.head_middle = ret.head_end = NULL;
   ret.body = NULL;
   ret.body_start = NULL;
   ret.body_end = NULL;
+  ret.meta_append = NULL;
   ret.address_start = NULL;
   ret.address_end = NULL;
   ret.nav_attrs = NULL;
@@ -164,47 +197,54 @@ static xhtmlconfig xhtml_configure(paragraph * source)
   ret.fsect[1].number_suffix = ustrdup(L" ");
   ret.rlink_prefix = NULL;
   ret.rlink_suffix = NULL;
+  ret.keywordfragments = TRUE;
 
   for (; source; source = source->next)
   {
     if (source->type == para_Config)
     {
-      if (!ustricmp(source->keyword, L"xhtml-contents-depth-0"))
+      if (ishtmlkeyword(source->keyword, L"html-version"))
+      {
+        const wchar_t* v = uadv(source->keyword);
+        if (!ustricmp(v, L"html4")) g_outputtype = OT_HTML4; // Note: Upstream treats this as "W3C HTML 4.01 Strict"
+        else if (!ustricmp(v, L"html5")) g_outputtype = OT_HTML5;
+        else if (!ustricmp(v, L"xhtml1.0transitional")) g_outputtype = OT_XHTML;
+        else error(err_whatever, "%ls unknown %ls", source->keyword, v);
+      } else if (ishtmlkeyword(source->keyword, L"xhtml-contents-depth-0"))
       {
         ret.contents_depth[0] = utoi(uadv(source->keyword));
-      } else if (!ustricmp(source->keyword, L"xhtml-contents-depth-1"))
+      } else if (ishtmlkeyword(source->keyword, L"xhtml-contents-depth-1"))
       {
         ret.contents_depth[1] = utoi(uadv(source->keyword));
-      } else if (!ustricmp(source->keyword, L"xhtml-contents-depth-2"))
+      } else if (ishtmlkeyword(source->keyword, L"xhtml-contents-depth-2"))
       {
         ret.contents_depth[2] = utoi(uadv(source->keyword));
-      } else if (!ustricmp(source->keyword, L"xhtml-contents-depth-3"))
+      } else if (ishtmlkeyword(source->keyword, L"xhtml-contents-depth-3"))
       {
         ret.contents_depth[3] = utoi(uadv(source->keyword));
-      } else if (!ustricmp(source->keyword, L"xhtml-contents-depth-4"))
+      } else if (ishtmlkeyword(source->keyword, L"xhtml-contents-depth-4"))
       {
         ret.contents_depth[4] = utoi(uadv(source->keyword));
-      } else if (!ustricmp(source->keyword, L"xhtml-contents-depth-5"))
+      } else if (ishtmlkeyword(source->keyword, L"xhtml-contents-depth-5"))
       {
         ret.contents_depth[5] = utoi(uadv(source->keyword));
-      } else if (!ustricmp(source->keyword, L"xhtml-leaf-level"))
+      } else if (ishtmlkeyword(source->keyword, L"xhtml-leaf-level"))
       {
         ret.leaf_level = utoi(uadv(source->keyword));
       } else
-          if (!ustricmp(source->keyword, L"xhtml-leaf-smallest-contents"))
+          if (ishtmlkeyword(source->keyword, L"xhtml-leaf-smallest-contents"))
       {
         ret.leaf_smallest_contents = utoi(uadv(source->keyword));
-      } else if (!ustricmp(source->keyword, L"xhtml-versionid"))
+      } else if (ishtmlkeyword(source->keyword, L"xhtml-versionid"))
       {
         ret.include_version_id = utob(uadv(source->keyword));
-      } else
-          if (!ustricmp(source->keyword, L"xhtml-leaf-contains-contents"))
+      } else if (ishtmlkeyword(source->keyword, L"xhtml-leaf-contains-contents"))
       {
         ret.leaf_contains_contents = utob(uadv(source->keyword));
-      } else if (!ustricmp(source->keyword, L"xhtml-suppress-address"))
+      } else if (ishtmlkeyword(source->keyword, L"xhtml-suppress-address"))
       {
         ret.suppress_address = utob(uadv(source->keyword));
-      } else if (!ustricmp(source->keyword, L"xhtml-author"))
+      } else if (ishtmlkeyword(source->keyword, L"xhtml-author"))
       {
         ret.author = uadv(source->keyword);
       } else if (!ustricmp(source->keyword, L"chm-toc-file"))
@@ -213,44 +253,49 @@ static xhtmlconfig xhtml_configure(paragraph * source)
       } else if (!ustricmp(source->keyword, L"chm-ind-file"))
       {
         ret.chm_ind_file = uadv(source->keyword);
-      } else if (!ustricmp(source->keyword, L"xhtml-description"))
+      } else if (ishtmlkeyword(source->keyword, L"xhtml-description"))
       {
         ret.description = uadv(source->keyword);
-      } else if (!ustricmp(source->keyword, L"xhtml-head-end"))
-      {
-        ret.head_end = uadv(source->keyword);
-      } else if (!ustricmp(source->keyword, L"xhtml-body-start"))
+      } else if (configurekeyword(&ret.html_lang, L"xhtml-lang", source)) {
+      } else if (configurekeyword(&ret.meta_charset, L"xhtml-meta-charset", source)) {
+      } else if (configurekeyword(&ret.head_start, L"xhtml-head-start", source)) {
+      } else if (configurekeyword(&ret.head_middle, L"xhtml-head-middle", source)) {
+      } else if (configurekeyword(&ret.head_end, L"xhtml-head-end", source)) {
+      } else if (ishtmlkeyword(source->keyword, L"xhtml-body-start"))
       {
         ret.body_start = uadv(source->keyword);
-      } else if (!ustricmp(source->keyword, L"xhtml-body-tag"))
+      } else if (ishtmlkeyword(source->keyword, L"xhtml-body-tag"))
       {
         ret.body = uadv(source->keyword);
-      } else if (!ustricmp(source->keyword, L"xhtml-body-end"))
+      } else if (ishtmlkeyword(source->keyword, L"xhtml-body-end"))
       {
         ret.body_end = uadv(source->keyword);
-      } else if (!ustricmp(source->keyword, L"xhtml-address-start"))
+      } else if (ishtmlkeyword(source->keyword, L"html-append-meta"))
+      {
+        ustr_slist_append(&ret.meta_append, uadv(source->keyword));
+      } else if (ishtmlkeyword(source->keyword, L"xhtml-address-start"))
       {
         ret.address_start = uadv(source->keyword);
-      } else if (!ustricmp(source->keyword, L"xhtml-address-end"))
+      } else if (ishtmlkeyword(source->keyword, L"xhtml-address-end"))
       {
         ret.address_end = uadv(source->keyword);
       } else
-          if (!ustricmp(source->keyword, L"xhtml-navigation-attributes"))
+          if (ishtmlkeyword(source->keyword, L"xhtml-navigation-attributes"))
       {
         ret.nav_attrs = uadv(source->keyword);
-      } else if (!ustricmp(source->keyword, L"xhtml-chapter-numeric"))
+      } else if (ishtmlkeyword(source->keyword, L"xhtml-chapter-numeric"))
       {
         ret.fchapter.just_numbers = utob(uadv(source->keyword));
-      } else if (!ustricmp(source->keyword, L"xhtml-chapter-suffix"))
+      } else if (ishtmlkeyword(source->keyword, L"xhtml-chapter-suffix"))
       {
-        ret.fchapter.number_suffix = ustrdup(uadv(source->keyword));
-      } else if (!ustricmp(source->keyword, L"xhtml-rlink-prefix"))
+        ustrreplacedup(&ret.fchapter.number_suffix, uadv(source->keyword));
+      } else if (ishtmlkeyword(source->keyword, L"xhtml-rlink-prefix"))
       {
         ret.rlink_prefix = uadv(source->keyword);
-      } else if (!ustricmp(source->keyword, L"xhtml-rlink-suffix"))
+      } else if (ishtmlkeyword(source->keyword, L"xhtml-rlink-suffix"))
       {
         ret.rlink_suffix = uadv(source->keyword);
-      } else if (!ustricmp(source->keyword, L"xhtml-section-numeric"))
+      } else if (ishtmlkeyword(source->keyword, L"xhtml-section-numeric"))
       {
         wchar_t *p = uadv(source->keyword);
         int n = 0;
@@ -268,7 +313,7 @@ static xhtmlconfig xhtml_configure(paragraph * source)
           ret.nfsect = n + 1;
         }
         ret.fsect[n].just_numbers = utob(p);
-      } else if (!ustricmp(source->keyword, L"xhtml-section-suffix"))
+      } else if (ishtmlkeyword(source->keyword, L"xhtml-section-suffix"))
       {
         wchar_t *p = uadv(source->keyword);
         int n = 0;
@@ -285,7 +330,10 @@ static xhtmlconfig xhtml_configure(paragraph * source)
             ret.fsect[i] = ret.fsect[ret.nfsect - 1];
           ret.nfsect = n + 1;
         }
-        ret.fsect[n].number_suffix = ustrdup(p);
+        ustrreplacedup(&ret.fsect[n].number_suffix, p);
+      } else if (ishtmlkeyword(source->keyword, L"xhtml-keywordfragments"))
+      {
+        ret.keywordfragments = utob(uadv(source->keyword));
       }
     }
   }
@@ -301,6 +349,9 @@ static xhtmlconfig xhtml_configure(paragraph * source)
   return ret;
 }
 
+#define element_close(rs__, tagname__) rdaddsc((rs__), "</" tagname__ ">")
+#define element_open(rs__, tagname__) rdaddsc((rs__), "<" tagname__ ">")
+
 static xhtmlsection *xhtml_new_section(xhtmlsection * last)
 {
   xhtmlsection *ret = mknew(xhtmlsection);
@@ -665,6 +716,39 @@ static void xhtml_ponder_layout(paragraph * p)
   xhtml_fixup_layout(topfile);  /* leaf files not at leaf level marked as such */
 }
 
+static void xhtml_hack_xhtmlify(word*words) // "\" "<br" "\" ">" --> "\" "<br /" "\" ">"
+{
+  int prevwasslash = 0;
+  if (!is_xhtml()) return ;
+  for (;words; words = words->next)
+  {
+    if (!words->text) continue;
+    if (prevwasslash)
+    {
+      if (words->text[0] == '<' && !ustricmp(words->text, L"<br") && words->next && words->next->text && words->next->text[0] == '\\')
+        ustrreplacedup(&words->text, L"<br /");
+    }
+    else
+      prevwasslash = words->text[0] == '\\' && !words->text[1];
+  }
+}
+
+#define NAMEDFRAGMENT_MAXLEN 200 /* More than enough for our usage */
+/*
+ * Get formatted fragment name for html anchor.
+ * Uses para->keyword if possible, falls back to the ?#.#.# default.
+ */
+static char*
+xhtml_get_fragmentname(const xhtmlsection * section, char * fragmentbuf)
+{
+  if (conf.keywordfragments)
+  {
+    paragraph *para = section->para;
+    if (para && para->keyword && *para->keyword) return ustrtoa(para->keyword, fragmentbuf, NAMEDFRAGMENT_MAXLEN);
+  }
+  return section->fragment;
+}
+
 static void xhtml_do_index();
 static void xhtml_do_file(xhtmlfile * file);
 static void xhtml_do_top_file(xhtmlfile * file, paragraph * sourceform);
@@ -783,6 +867,7 @@ xhtml_backend(paragraph * sourceform, keywordlist * in_keywords,
       sfree(conf.fsect[i].number_suffix);
     sfree(conf.fsect);
   }
+  free_ustr_slist(conf.meta_append);
 }
 
 static int xhtml_para_level(paragraph * p)
@@ -888,8 +973,9 @@ static void xhtml_do_index_body(FILE * fp)
         xhtmlsection *sect = xi->sections[i];
         if (sect)
         {
-          fprintf(fp, "<a href='%s#%s'>", sect->file->filename,
-                  sect->fragment);
+          char fragmentbuf[NAMEDFRAGMENT_MAXLEN];
+          fprintf(fp, "<a href='%s#%s'>", conf.leaf_level ? sect->file->filename : "",
+                  xhtml_get_fragmentname(sect, fragmentbuf));
           if (sect->para->kwtext)
           {
             xhtml_para(fp, sect->para->kwtext);
@@ -973,6 +1059,8 @@ static void xhtml_do_top_file(xhtmlfile * file, paragraph * sourceform)
   if (fp == NULL)
     fatal(err_cantopenw, file->filename);
 
+  if (conf.chm_toc_file || conf.chm_ind_file) g_outputtype = OT_CHM | (g_outputtype & (OT_XHTML|OT_HTML5));
+
   ustrtoa(conf.chm_toc_file, fname, 4096);
   if(*fname)
   {
@@ -1024,6 +1112,7 @@ static void xhtml_do_top_file(xhtmlfile * file, paragraph * sourceform)
     if (p->type == para_Preamble)
     {
       fprintf(fp, "<p>");
+      xhtml_hack_xhtmlify(p->words);
       xhtml_para(fp, p->words);
       fprintf(fp, "</p>\n");
     }
@@ -1033,6 +1122,7 @@ static void xhtml_do_top_file(xhtmlfile * file, paragraph * sourceform)
     if (p->type == para_Copyright)
     {
       fprintf(fp, "<p>");
+      xhtml_hack_xhtmlify(p->words);
       xhtml_para(fp, p->words);
       fprintf(fp, "</p>\n");
     }
@@ -1205,6 +1295,7 @@ xhtml_do_contents_section_limit(FILE * fp, xhtmlsection * section, int limit)
 static int
 xhtml_add_contents_entry(FILE * fp, xhtmlsection * section, int limit)
 {
+  char fragmentbuf[NAMEDFRAGMENT_MAXLEN], *fragment, *filename;
   if (!section || section->level > limit)
     return FALSE;
   if (fp == NULL || section->level < 0)
@@ -1221,17 +1312,19 @@ xhtml_add_contents_entry(FILE * fp, xhtmlsection * section, int limit)
     fprintf(fp, "<ul>\n");
     if(chm_toc)fprintf(chm_toc, "<ul>\n");
   }
+  filename = conf.leaf_level ? section->file->filename : "";
+  fragment = "";
+  if (section->para->type != para_Chapter || !conf.leaf_level)
+    fragment = xhtml_get_fragmentname(section, fragmentbuf);
+
   fprintf(fp, "<li>");
   fprintf(fp, "<a %shref=\"%s#%s\">",
           (section->para->type == para_Chapter|| section->para->type == para_Appendix) ? "class=\"btitle\" " : "",
-          section->file->filename,
-          (section->para->type == para_Chapter) ? "" : section->fragment);
+          filename, fragment);
   if(chm_toc)fprintf(chm_toc, "<li><OBJECT type=\"text/sitemap\"><param name=\"Local\" value=\"%s#%s\"><param name=\"Name\" value=\"",
-          section->file->filename,
-          (section->para->type == para_Chapter) ? "" : section->fragment);
+          filename, fragment);
   if(chm_ind)fprintf(chm_ind, "<li><OBJECT type=\"text/sitemap\"><param name=\"Local\" value=\"%s#%s\"><param name=\"Name\" value=\"",
-          section->file->filename,
-          (section->para->type == para_Chapter) ? "" : section->fragment);
+          filename, fragment);
           //%s
   if (section->para->type == para_Chapter
       || section->para->type == para_Appendix)
@@ -1320,7 +1413,7 @@ static void xhtml_do_paras(FILE * fp, paragraph * p)
       break;
 
     case para_Rule:
-      fprintf(fp, "\n<hr />\n");
+      fprintf(fp, "\n%s\n", gettagtxt_hr());
       break;
 
     case para_Normal:
@@ -1401,37 +1494,54 @@ static void xhtml_do_paras(FILE * fp, paragraph * p)
   }
 }
 
+static void printoptstr(FILE * fp, const char*prefix, const wchar_t*str, const char*suffix)
+{
+  if (str)
+  {
+    fprintf(fp, "%s%ls%s", prefix ? prefix : "", str, suffix ? suffix : "");
+  }
+}
+
 /*
  * Output a header for this XHTML file.
  */
 static void xhtml_doheader(FILE * fp, word * title)
 {
-  fprintf(fp,
-          "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\"\n");
-  fprintf(fp,
-          "\"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n");
-  fprintf(fp,
-          "<html xmlns='http://www.w3.org/1999/xhtml'>\n\n<head>\n<title>");
+  const int xhtml = is_xhtml(), html5 = is_html5();
+  const char *xhtmldoctype = "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n";
+  const char *html4doctype = "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">\n";
+  const char *xhtmlxmlns = xhtml ? " xmlns=\"http://www.w3.org/1999/xhtml\"" : 0;
+  const char *voidend = xhtml ? " /" : "";
+  const wchar_t *tmpwstr;
+  const ustr_slist *pussl;
+
+  if (xhtml && html5) fatal(err_whatever, "indeterminate format");
+  fprintf(fp, html5 ? "<!DOCTYPE html>\n" : xhtml ? xhtmldoctype : html4doctype);
+  fprintf(fp, "<html%s", xhtml ? xhtmlxmlns : "");
+  //www.w3.org/International/questions/qa-html-language-declarations
+  if (*(tmpwstr = ustrdef(conf.html_lang, L"")))
+    fprintf(fp, "%s%ls%s lang=\"%ls\"", xhtml ? " xml:lang=\"" : "", xhtml ? tmpwstr : L"", xhtml ? "\"" : "", tmpwstr);
+  fprintf(fp, "><head>\n");
+  if (ustricmp(L"none", (tmpwstr = ustrdef(conf.meta_charset, L"UTF-8"))))
+    fprintf(fp, (xhtml || !html5) ? "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=%ls\"%s>" : "<meta charset=\"%ls\">\n", tmpwstr, voidend);
+  printoptstr(fp, "", conf.head_start, "\n");
+  fprintf(fp, "<title>");
   if (title == NULL)
     fprintf(fp, "Documentation");
   else
     xhtml_para(fp, title);
   fprintf(fp, "</title>\n");
-  fprintf(fp,
-          "<meta name=\"generator\" content=\"Halibut %s xhtml-backend\" />\n",
-          version);
+  for (pussl = conf.meta_append; pussl; pussl = pussl->next)
+    fprintf(fp, "<meta %ls%s>", pussl->string, voidend);
+  printoptstr(fp, "", conf.head_middle, "\n");
+  fprintf(fp, "<meta name=\"generator\" content=\"Halibut %s xhtml-backend\"%s>\n", version, voidend);
   if (conf.author)
-    fprintf(fp, "<meta name=\"author\" content=\"%ls\" />\n", conf.author);
+    fprintf(fp, "<meta name=\"author\" content=\"%ls\"%s>\n", conf.author, voidend);
   if (conf.description)
-    fprintf(fp, "<meta name=\"description\" content=\"%ls\" />\n",
-            conf.description);
-  if (conf.head_end)
-    fprintf(fp, "%ls\n", conf.head_end);
-  fprintf(fp, "</head>\n\n");
-  if (conf.body)
-    fprintf(fp, "%ls\n", conf.body);
-  else
-    fprintf(fp, "<body>\n");
+    fprintf(fp, "<meta name=\"description\" content=\"%ls\"%s>\n", conf.description, voidend);
+  printoptstr(fp, "", conf.head_end, "\n");
+  fprintf(fp, "</head>\n");
+  fprintf(fp, "%ls\n", conf.body ? conf.body : L"<body>");
   if (conf.body_start)
     fprintf(fp, "%ls\n", conf.body_start);
 }
@@ -1448,7 +1558,11 @@ static void chm_doheader(FILE * fp, word * title)
  */
 static void xhtml_dofooter(FILE * fp)
 {
-  fprintf(fp, "\n<hr />\n\n");
+  int hr = conf.body_end || !conf.suppress_address;
+
+  if (hr)
+    fprintf(fp, "\n%s\n\n", gettagtxt_hr());
+
   if (conf.body_end)
     fprintf(fp, "%ls\n", conf.body_end);
   if (!conf.suppress_address)
@@ -1472,7 +1586,7 @@ static void xhtml_dofooter(FILE * fp)
       fprintf(fp, "%ls\n", conf.address_end);
     fprintf(fp, "</address>\n");
   }
-  fprintf(fp, "</body>\n\n</html>\n");
+  fprintf(fp, "</body></html>\n");
 }
 static void chm_dofooter(FILE * fp)
 {
@@ -1492,7 +1606,7 @@ static void xhtml_versionid(FILE * fp, word * text, int started)
   rdaddc(&t, ']');              /* FIXME: configurability */
 
   if (started)
-    fprintf(fp, "<br>\n");
+    fprintf(fp, "%s\n", gettagtxt_br());
   fprintf(fp, "%s\n", t.text);
   sfree(t.text);
 }
@@ -1626,7 +1740,7 @@ static void xhtml_rdaddwc(rdstringc * rs, word * text, word * end)
   keyword *kwl;
   xhtmlsection *sect;
   indextag *itag;
-  int ti;
+  int ti, style, type;
   wchar_t *s;
 
   for (; text && text != end; text = text->next)
@@ -1639,7 +1753,7 @@ static void xhtml_rdaddwc(rdstringc * rs, word * text, word * end)
       if(chm_toc && *c == '.' && *(c+1) == '.')
         rdaddsc(rs, c + 1);
       else
-	      rdaddsc(rs, c);
+        rdaddsc(rs, c);
       rdaddsc(rs, "\">");
       sfree(c);
       break;
@@ -1654,7 +1768,7 @@ static void xhtml_rdaddwc(rdstringc * rs, word * text, word * end)
         rdaddsc(rs, c2);
         sfree(c2);
       }
-	    rdaddsc(rs, c);
+      rdaddsc(rs, c);
       if (conf.rlink_suffix)
       {
         char *c2;
@@ -1675,10 +1789,11 @@ static void xhtml_rdaddwc(rdstringc * rs, word * text, word * end)
         sect = xhtml_find_section(kwl->para);
         if (sect)
         {
+          char fragmentbuf[NAMEDFRAGMENT_MAXLEN];
           rdaddsc(rs, "<a href=\"");
-          rdaddsc(rs, sect->file->filename);
+          rdaddsc(rs, conf.leaf_level ? sect->file->filename : "");
           rdaddc(rs, '#');
-          rdaddsc(rs, sect->fragment);
+          rdaddsc(rs, xhtml_get_fragmentname(sect, fragmentbuf));
           rdaddsc(rs, "\">");
         } else
         {
@@ -1775,29 +1890,43 @@ static void xhtml_rdaddwc(rdstringc * rs, word * text, word * end)
       rdaddsc(rs, "</a>");
       break;
 
+    case word_Html:
+      xhtml_utostr(text->text, &c);
+      rdaddsc(rs, c);
+      sfree(c);
+      break;
+
     case word_Normal:
     case word_Emph:
+    case word_Strong:
+    case word_Bold:
     case word_Code:
     case word_WeakCode:
     case word_WhiteSpace:
     case word_EmphSpace:
+    case word_StrongSpace:
+    case word_BoldSpace:
     case word_CodeSpace:
     case word_WkCodeSpace:
     case word_Quote:
     case word_EmphQuote:
+    case word_StrongQuote:
+    case word_BoldQuote:
     case word_CodeQuote:
     case word_WkCodeQuote:
       assert(text->type != word_CodeQuote &&
              text->type != word_WkCodeQuote);
-      if (towordstyle(text->type) == word_Emph &&
-          (attraux(text->aux) == attr_First ||
-           attraux(text->aux) == attr_Only))
-        rdaddsc(rs, "<em>");
-      else if ((towordstyle(text->type) == word_Code
-                || towordstyle(text->type) == word_WeakCode)
-               && (attraux(text->aux) == attr_First
-                   || attraux(text->aux) == attr_Only))
-        rdaddsc(rs, "<code>");
+      style = towordstyle(text->type);
+      type = removeattr(text->type);
+
+      switch((attraux(text->aux) == attr_First || attraux(text->aux) == attr_Only) ? style : word_NotWordType)
+      {
+      case word_Emph: element_open(rs, "em"); break;
+      case word_Strong: element_open(rs, "strong"); break;
+      case word_Bold: element_open(rs, "b"); break;
+      case word_Code: element_open(rs, "code"); break;
+      case word_WeakCode: element_open(rs, "code"); break;
+      }
 
       if (removeattr(text->type) == word_Normal)
       {
@@ -1839,15 +1968,14 @@ static void xhtml_rdaddwc(rdstringc * rs, word * text, word * end)
         rdaddsc(rs, "&quot;");
       }
 
-      if (towordstyle(text->type) == word_Emph &&
-          (attraux(text->aux) == attr_Last ||
-           attraux(text->aux) == attr_Only))
-        rdaddsc(rs, "</em>");
-      else if ((towordstyle(text->type) == word_Code
-                || towordstyle(text->type) == word_WeakCode)
-               && (attraux(text->aux) == attr_Last
-                   || attraux(text->aux) == attr_Only))
-        rdaddsc(rs, "</code>");
+      switch((attraux(text->aux) == attr_Last || attraux(text->aux) == attr_Only) ? style : word_NotWordType)
+      {
+      case word_Emph: element_close(rs, "em"); break;
+      case word_Strong: element_close(rs, "strong"); break;
+      case word_Bold: element_close(rs, "b"); break;
+      case word_Code: element_close(rs, "code"); break;
+      case word_WeakCode: element_close(rs, "code"); break;
+      }
       break;
     }
   }
@@ -1864,10 +1992,10 @@ static void xhtml_heading(FILE * fp, paragraph * p)
   int level = xhtml_para_level(p);
   xhtmlsection *sect = xhtml_find_section(p);
   xhtmlheadfmt *fmt;
-  char *fragment;
+  char fragmentbuf[NAMEDFRAGMENT_MAXLEN], *fragment;
   if (sect)
   {
-    fragment = sect->fragment;
+    fragment = xhtml_get_fragmentname(sect, fragmentbuf);
   } else
   {
     if (p->type == para_Title)
