@@ -3,7 +3,7 @@
  * 
  * This file is a part of NSIS.
  * 
- * Copyright (C) 1999-2020 Nullsoft and Contributors
+ * Copyright (C) 1999-2021 Nullsoft and Contributors
  * 
  * Licensed under the zlib/libpng license (the "License");
  * you may not use this file except in compliance with the License.
@@ -634,6 +634,10 @@ typedef DWORDLONG ULONGLONG,*PULONGLONG;
 #  define PBS_SMOOTH 1
 #endif
 
+#ifndef SS_REALSIZECONTROL
+#  define SS_REALSIZECONTROL 0x0040
+#endif
+
 #ifndef DS_SHELLFONT
 #  define DS_SHELLFONT (DS_SETFONT | DS_FIXEDSYS)
 #endif
@@ -723,6 +727,9 @@ typedef DWORDLONG ULONGLONG,*PULONGLONG;
 #ifndef DEFAULT_CHARSET
 #  define DEFAULT_CHARSET 1
 #endif
+#ifndef SHIFTJIS_CHARSET
+#  define SHIFTJIS_CHARSET 128
+#endif
 #ifndef OUT_DEFAULT_PRECIS
 #  define OUT_DEFAULT_PRECIS 0
 #endif
@@ -760,6 +767,9 @@ typedef DWORDLONG ULONGLONG,*PULONGLONG;
 #  define FILE_ATTRIBUTE_TEMPORARY 0x00000100
 #  define FILE_ATTRIBUTE_OFFLINE 0x00001000
 #endif
+#ifndef FILE_ATTRIBUTE_NOT_CONTENT_INDEXED
+#  define FILE_ATTRIBUTE_NOT_CONTENT_INDEXED 0x002000
+#endif
 
 
 // fopen
