@@ -3,7 +3,7 @@
  * 
  * This file is a part of NSIS.
  * 
- * Copyright (C) 2002-2020 Amir Szekely <kichik@users.sourceforge.net>
+ * Copyright (C) 2002-2021 Amir Szekely <kichik@users.sourceforge.net>
  * 
  * Licensed under the zlib/libpng license (the "License");
  * you may not use this file except in compliance with the License.
@@ -30,6 +30,7 @@ using namespace std;
 // Utilities
 //////////////////////////////////////////////////////////////////////
 
+#define FIRSTRESDIRSTRADDRESS ( (WINWCHAR*)(~(size_t)0) )
 #define WCHARPTR2WINWCHARPTR(s) ( (WINWCHAR*) (s) ) // Only for WinSDK structs like IMAGE_RESOURCE_DIR_STRING_U where we cannot change the WCHAR type!
 #define RALIGN(dwToAlign, dwAlignOn) ((dwToAlign%dwAlignOn == 0) ? dwToAlign : dwToAlign - (dwToAlign%dwAlignOn) + dwAlignOn)
 #define ALIGN(dwToAlign, dwAlignOn) dwToAlign = RALIGN((dwToAlign), (dwAlignOn))
@@ -37,6 +38,31 @@ using namespace std;
 static inline DWORD ConvertEndianness(DWORD d) { return FIX_ENDIAN_INT32(d); }
 static inline WORD ConvertEndianness(WORD w) { return FIX_ENDIAN_INT16(w); }
 
+#if !(defined(_WIN32) && defined(_UNICODE))
+static void FreeUnicodeResString(WINWCHAR* s) {
+  if (!IS_INTRESOURCE(s) && FIRSTRESDIRSTRADDRESS != (WINWCHAR*) s)
+    free(s);
+}
+static WINWCHAR* ResStringToUnicode(const TCHAR *s) {
+  if (IS_INTRESOURCE(s)) return MAKEINTRESOURCEWINW((ULONG_PTR)s);
+  if (FIRSTRESDIRSTRADDRESS == (WINWCHAR*) s) return (WINWCHAR*) s;
+  WINWCHAR *ws = WinWStrDupFromTChar(s);
+  if (!ws) throw std::runtime_error("Unicode conversion failed");
+  return ws;
+}
+#endif //~ !(_WIN32 && _UNICODE)
+
+struct UTF16LEResString {
+  WINWCHAR *m_s;
+  operator WINWCHAR*() const { return m_s; }
+#if defined(_WIN32) && defined(_UNICODE)
+  UTF16LEResString(const TCHAR*tstr) : m_s((WINWCHAR*) tstr) {}
+#else
+  UTF16LEResString(const TCHAR*tstr) : m_s(ResStringToUnicode(tstr)) { }
+  ~UTF16LEResString() { FreeUnicodeResString(m_s); }
+#endif
+};
+
 PIMAGE_NT_HEADERS CResourceEditor::GetNTHeaders(BYTE* pbPE) {
   // Get dos header
   PIMAGE_DOS_HEADER dosHeader = (PIMAGE_DOS_HEADER) pbPE;
@@ -140,12 +166,13 @@ const TCHAR* CResourceEditor::ParseResourceTypeString(const TCHAR*Str)
 
 #include "exehead/resource.h" // IDI_ICON2
 #include "exehead/config.h" // NSIS_DEFAULT_LANG
-const TCHAR* CResourceEditor::ParseResourceNameString(const TCHAR*Str) {
+const TCHAR* CResourceEditor::ParseResourceNameString(const TCHAR*Str, bool AllowFirst) {
   if (Str[0] == _T('#'))
   {
     ++Str;
     if (!_tcsicmp(Str, _T("Version"))) return (TCHAR*) MAKEINTRESOURCE(1);
     if (!_tcsicmp(Str, _T("Icon"))) return (TCHAR*) MAKEINTRESOURCE(IDI_ICON2);
+    if (AllowFirst && Str[0] == '?' && !Str[1]) return (TCHAR*) FIRSTRESDIRSTRADDRESS;
     int succ, num = ParseSimpleInt(Str, &succ);
     return succ && IS_INTRESOURCE(num) ? (TCHAR*) MAKEINTRESOURCE(num) : NULL;
   }
@@ -161,20 +188,81 @@ LANGID CResourceEditor::ParseResourceLangString(const TCHAR*Str) {
   return succ ? num : INVALIDLANGID;
 }
 
-LANGID CResourceEditor::ParseResourceTypeNameLangString(const TCHAR**Type, const TCHAR**Name, const TCHAR*Lang) {
+LANGID CResourceEditor::ParseResourceTypeNameLangString(const TCHAR**Type, const TCHAR**Name, const TCHAR*Lang, bool AllowFirst) {
   if (!(*Type = ParseResourceTypeString(*Type))) return INVALIDLANGID;
-  if (!(*Name = ParseResourceNameString(*Name))) return INVALIDLANGID;
+  if (!(*Name = ParseResourceNameString(*Name, AllowFirst))) return INVALIDLANGID;
   return ParseResourceLangString(Lang);
 }
 
-template<class T> static WORD GetDependentType(T Type)
+UINT CResourceEditor::IsResProtocol(const TCHAR*Url)
 {
+  if ('r' == S7ChLwr(Url[0]) && 'e' == S7ChLwr(Url[1]) && 's' == S7ChLwr(Url[2]))
+    if (':' == Url[3] && '/' == Url[4] && '/' == Url[5])
+      return 6;
+  return 0;
+}
+
+static TCHAR* ParseResProtocolAlloc(const TCHAR*Url, const TCHAR*&Type, const TCHAR*&Name, LANGID&Lang) {
+  //msdn.microsoft.com/library/aa767740#res Protocol
+  TCHAR *path = 0, *buf = 0, *pD, ch;
+  UINT prefix, mswin = Platform_IsWindows(), bad = false, pipe = 0, skip = 0;
+  size_t typestart = 0, namestart = 0, i, cch;
+  if (!(prefix = CResourceEditor::IsResProtocol(Url)))
+    return path;
+  for (Url += prefix, i = 0; Url[i]; ++i)
+    if (Url[i] == '/')
+      typestart = namestart, namestart = i;
+  const TCHAR*pS = Url;
+  if (namestart > 2 && (buf = (TCHAR*) malloc((cch = ++i) * sizeof(*Url)))) {
+    if (pS[0] == '/') ++pS;
+    ch = S7ChLwr(pS[0]);
+    if (ch >= 'a' && ch <= 'z' && (pS[1] == ':' || pS[1] == '|') && IsAgnosticPathSeparator(pS[2])) { // IEBlog:"File URIs in Windows" says %3A is not a drive delimiter.
+      if (Url[0] == '/') ++skip; // "res:///C:/.." => "res://C:/.." (Even on POSIX so that our FOPEN can do "c:/.." => "/c/..")
+      pipe = (UINT)(size_t) ((pS + 1) - (Url + skip));
+    }
+    typestart -= skip, namestart -= skip;
+    const TCHAR *rt = buf + typestart + 1, *rn = buf + namestart + 1, *rl = _T("Any");
+    my_strncpy(buf, Url + skip, cch);
+    buf[typestart] = buf[namestart] = '\0'; // Note: Type and Name are not decoded.
+    if (pipe) buf[pipe] = ':'; // "res://C|/.." => "res://C:/.." (The | replacement is technically a file:// legacy feature but we support it for res:// as well)
+    for (pD = buf, pS = pD;; ++pS, ++pD) {
+      if ((ch = *pS) == '%') { // Deal with percent-encoding
+        if (*++pS != '%') {
+          TCHAR hex[3] = { pS[0], pS[0] ? pS[1] : _T('\0'), _T('\0') };
+          ch = ChIsHex(pS[0]) && ChIsHex(pS[1]) ? (TCHAR) _tcstol(hex, 0, 16) : 0;
+          if (ch) ++pS; else ++bad;
+        }
+      }
+      if (!(*pD = (mswin && ch == '/') ? '\\' : ch)) break; // Convert path if needed and stop at the end.
+    }
+    Lang = CResourceEditor::ParseResourceTypeNameLangString(&rt, &rn, rl, true);
+    if (!bad && Lang != CResourceEditor::INVALIDLANGID)
+      path = buf, Type = rt, Name = rn;
+  }
+  if (!path) free(buf);
+  return path;
+}
+
+template<class T> static WORD GetDependentType(T Type) {
   if (!IS_INTRESOURCE((size_t) Type)) return 0;
   if (MAKEINTRESOURCE((size_t) Type) == RT_GROUP_ICON) return (WORD)(size_t) RT_ICON;
   if (MAKEINTRESOURCE((size_t) Type) == RT_GROUP_CURSOR) return (WORD)(size_t) RT_CURSOR;
   return 0;
 }
 
+template<class T> static WORD IsIcoCurSingleImageType(T Type) {
+  WORD t = IS_INTRESOURCE((size_t) Type) ? (WORD)(size_t) Type : 0;
+  return t == (WORD)(size_t) RT_ICON || t == (WORD)(size_t) RT_CURSOR;
+}
+
+template<class T> static WORD IsIcoCurGroupType(T Type) {
+  return IsIcoCurSingleImageType(GetDependentType(Type));
+}
+
+template<class T> static WORD IsIcoCurType(T Type) {
+  return IsIcoCurSingleImageType(Type) || IsIcoCurGroupType(Type);
+}
+
 //////////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////////
 // CResourceEditor
@@ -191,6 +279,9 @@ CResourceEditor::CResourceEditor(void* pbPE, int iSize, bool bKeepData /*=true*/
   m_iSize = iSize;
   m_bKeepData = bKeepData;
 
+  assert(!EditorSupportsStringNames());
+  assert(!EditorSupportsCursorPng());
+
   // Get NT headers
   m_ntHeaders = GetNTHeaders(m_pbPE);
 
@@ -218,26 +309,51 @@ CResourceEditor::~CResourceEditor() {
 // Methods
 //////////////////////////////////////////////////////////////////////
 
-#define FINDRESOURCE_NAME_FIRSTITEM ( (WINWCHAR*)(~(size_t)0) )
+bool CResourceEditor::CanOpen(const void*Data, size_t Size)
+{
+  return 'P' == GetExeType(Data, Size); // We only understand PE, not NE/LE
+}
 
-CResourceDirectoryEntry* CResourceEditor::FindResourceLanguageDirEntryW(const WINWCHAR* Type, const WINWCHAR* Name, LANGID Language) {
-  int i = m_cResDir->Find(Type);
-  if (-1 != i) {
-    CResourceDirectory* pND = m_cResDir->GetEntry(i)->GetSubDirectory();
-    i = FINDRESOURCE_NAME_FIRSTITEM == Name ? 0 : pND->Find(Name);
-    if (-1 != i) {
-      CResourceDirectory* pLD = pND->GetEntry(i)->GetSubDirectory();
-      i = ANYLANGID == Language ? 0 : pLD->Find(Language);
-      if (-1 != i)
-        return pLD->GetEntry(i);
+CResourceDataEntry* CResourceEditor::FindResourceW(const WINWCHAR*RT, const WINWCHAR*RN, LANGID RL, CResourceDirectoryEntry**ppTE, CResourceDirectoryEntry**ppNE, CResourceDirectoryEntry**ppLE) const {
+  int i = m_cResDir->Find(RT);
+  CResourceDirectoryEntry*pTDE = -1 != i ? m_cResDir->GetEntry(i) : 0, *pNDE, *pLDE;
+  if (pTDE) {
+    CResourceDirectory *pND = pTDE->GetSubDirectory();
+    i = FIRSTRESDIRSTRADDRESS == RN ? 0 : pND->Find(RN);
+    if ((pNDE = (-1 != i) ? pND->GetEntry(i) : 0)) {
+      CResourceDirectory *pLD = pNDE->GetSubDirectory();
+      i = ANYLANGID == RL ? 0 : pLD->Find(RL);
+      if ((pLDE = (-1 != i) ? pLD->GetEntry(i) : 0)) {
+        if (ppTE) *ppTE = pTDE;
+        if (ppNE) *ppNE = pNDE;
+        if (ppLE) *ppLE = pLDE;
+        return pLDE->GetDataEntry();
+      }
     }
   }
   return 0;
 }
 
-CResourceDataEntry* CResourceEditor::FindResource(const WINWCHAR* Type, const WINWCHAR* Name, LANGID Language) {
-  CResourceDirectoryEntry*pRDE = FindResourceLanguageDirEntryW(Type, Name, Language);
-  return pRDE ? pRDE->GetDataEntry() : 0;
+CResourceDataEntry* CResourceEditor::FindResourceT(const TCHAR*RT, const TCHAR*RN, LANGID RL, CResourceDirectoryEntry**ppTE, CResourceDirectoryEntry**ppNE, CResourceDirectoryEntry**ppLE) const {
+  return FindResourceW(UTF16LEResString(RT), UTF16LEResString(RN), RL, ppTE, ppNE, ppLE);
+}
+
+CResourceDirectoryEntry* CResourceEditor::FindResourceLanguageDirEntryW(const WINWCHAR* RT, const WINWCHAR* RN, LANGID RL) const {
+  CResourceDirectoryEntry*pLDE = 0;
+  FindResourceW(RT, RN, RL, 0, 0, &pLDE);
+  return pLDE;
+}
+
+CResourceDataEntry* CResourceEditor::FindResource(const WINWCHAR* RT, const WINWCHAR* RN, LANGID RL) const {
+  CResourceDirectoryEntry*pDE = FindResourceLanguageDirEntryW(RT, RN, RL);
+  return pDE ? pDE->GetDataEntry() : 0;
+}
+
+template<class T> static void UpdateManipulationType(CResourceEditor::TYPEMANIPULATION &Manip, const T* szType, const void*Data, size_t Size) {
+  WORD dependenttype = GetDependentType(szType);
+  if (Manip == CResourceEditor::TM_AUTO)
+    if (IsIcoCurSingleImageType(dependenttype) && IsICOCURFile(Data, Size))
+      Manip = CResourceEditor::TM_ICONFILE;
 }
 
 // Adds/Replaces/Removes a simple resource.
@@ -263,18 +379,19 @@ bool CResourceEditor::UpdateResourceW(const WINWCHAR* szType, WINWCHAR* szName,
   }
 
   bool deleteoperation = !lpData, success = true, handlecomplexicon = false;
+  UpdateManipulationType(Manip, szType, lpData, dwSize);
   WORD dependenttype = GetDependentType(szType);
 
   if (dependenttype && Manip != TM_RAW) {
-    if (Manip == TM_AUTO && ((size_t) szType == (size_t) RT_GROUP_ICON || (size_t) szType == (size_t) RT_GROUP_CURSOR))
-      Manip = TM_ICON;
+    if (Manip == TM_AUTO && IsIcoCurSingleImageType(dependenttype))
+      Manip = TM_ICON; // A non-TM_ICONFILE operation that is probably going to fail
 
     if (Manip & TM_ICON)
-      handlecomplexicon = true;
+      handlecomplexicon = true; // Group and images
 
     if (handlecomplexicon && !deleteoperation)
       if (Manip == TM_AUTO || (Manip & TM_ICONRSRC))
-        return false; // It is impossible to add a resource icon from a plain data buffer because it doesn't use offsets for the images
+        return false; // It is impossible to add a icon from a resource-based plain data buffer because it doesn't use offsets for the images
 
     if ((size_t) szType == (size_t) RT_GROUP_ICON && (size_t) szName == (size_t) IDI_ICON2)
       return false; // The main icon is special, don't allow high-level RT_GROUP_ICON updates to touch RT_ICON.
@@ -343,37 +460,13 @@ bool CResourceEditor::UpdateResourceW(const WINWCHAR* szType, WINWCHAR* szName,
   return success;
 }
 
-#ifndef _UNICODE
-static WINWCHAR* ResStringToUnicode(const char *szString) {
-  if (IS_INTRESOURCE(szString)) return MAKEINTRESOURCEWINW((ULONG_PTR)szString);
-  WINWCHAR *s = WinWStrDupFromTChar(szString);
-  if (!s) throw std::bad_alloc();
-  return s;
-}
-static void FreeUnicodeResString(WINWCHAR* szwString) {
-  if (!IS_INTRESOURCE(szwString)) free(szwString);
-}
-#else
-#ifndef _WIN32
-static WINWCHAR* ResStringToUnicode(const TCHAR *s) {
-  if (IS_INTRESOURCE(s)) return MAKEINTRESOURCEWINW((ULONG_PTR)s);
-  WCToUTF16LEHlpr cnv;
-  if (!cnv.Create(s)) throw std::runtime_error("Unicode conversion failed");
-  return (WINWCHAR*) cnv.Detach();
-}
-static void FreeUnicodeResString(WINWCHAR* s) {
-  if (!IS_INTRESOURCE(s)) free(s);
-}
-#endif // ~_WIN32
-#endif // ~_UNICODE
-
-CResourceDirectoryEntry* CResourceEditor::FindResourceLanguageDirEntryT(const TCHAR* Type, const TCHAR* Name, LANGID Language) {
-  assert(!EditorSupportsStringNames() && sizeof(Name));
+CResourceDirectoryEntry* CResourceEditor::FindResourceLanguageDirEntryT(const TCHAR* RT, const TCHAR* RN, LANGID RL) const {
+  assert(!EditorSupportsStringNames() && sizeof(RN));
 #if defined(_WIN32) && defined(_UNICODE)
-  return FindResourceLanguageDirEntryW((WINWCHAR*)Type, (WINWCHAR*)Name, Language);
+  return FindResourceLanguageDirEntryW((WINWCHAR*)RT, (WINWCHAR*)RN, RL);
 #else
-  WINWCHAR* szwType = ResStringToUnicode(Type); 
-  CResourceDirectoryEntry* result = FindResourceLanguageDirEntryW(szwType, (WINWCHAR*)Name, Language);
+  WINWCHAR* szwType = ResStringToUnicode(RT); 
+  CResourceDirectoryEntry* result = FindResourceLanguageDirEntryW(szwType, (WINWCHAR*)RN, RL);
   FreeUnicodeResString(szwType);
   return result;
 #endif
@@ -398,10 +491,6 @@ bool CResourceEditor::UpdateResourceT(const TCHAR* szType, WORD szName, LANGID w
   BYTE *data = alloc_and_read_file(Data, size);
   if (!data)
     return false;
-  WORD dependenttype = GetDependentType(szType);
-  if (Manip == TM_AUTO)
-    if (dependenttype && IsICOCURFile(data, size))
-      Manip = TM_ICONFILE;
   result = UpdateResourceT(szType, szName, wLanguage, data, size, Manip);
   free(data);
   return result;
@@ -489,7 +578,7 @@ DWORD CResourceEditor::GetResourceOffsetT(const TCHAR* szType, WORD szName, LANG
 
 // Returns a copy of the resource data from the first resource of a specific type
 BYTE* CResourceEditor::GetFirstResourceW(const WINWCHAR* szType, size_t&cbData) {
-  CResourceDataEntry *pDE = FindResource(szType, FINDRESOURCE_NAME_FIRSTITEM, ANYLANGID);
+  CResourceDataEntry *pDE = FindResource(szType, FIRSTRESDIRSTRADDRESS, ANYLANGID);
   if (pDE)
   {
     cbData = pDE->GetSize();
@@ -741,10 +830,11 @@ static WORD FindFreeIconImageId(CResourceEditor& re, WORD ImgType) {
   return 0;
 }
 
+typedef struct { BYTE Width, Height, Palette, Reserved; WORD Planes, BPP; UINT32 Size, Offset; } FILEICOGROUPENTRY;
+typedef struct { BYTE Width, Height, Palette, Reserved; WORD Planes, BPP, SizeLo, SizeHi, Id; } RSRCICOGROUPENTRY;
+typedef struct { WORD Width, Height;                    WORD Planes, BPP, SizeLo, SizeHi, Id; } RSRCCURGROUPENTRY; //msdn.microsoft.com/en-us/library/windows/desktop/ms648011(v=vs.85).aspx
+
 bool CResourceEditor::AddExtraIconFromFile(const WINWCHAR* Type, WINWCHAR* Name, LANGID LangId, BYTE* Data, DWORD Size) {
-  typedef struct { BYTE Width, Height, Palette, Reserved; WORD Planes, BPP; UINT32 Size, Offset; } FILEICOGROUPENTRY;
-  typedef struct { BYTE Width, Height, Palette, Reserved; WORD Planes, BPP, SizeLo, SizeHi, Id; } RSRCICOGROUPENTRY;
-  typedef struct { WORD Width, Height;                    WORD Planes, BPP, SizeLo, SizeHi, Id; } RSRCCURGROUPENTRY; //msdn.microsoft.com/en-us/library/windows/desktop/ms648011(v=vs.85).aspx
   assert(sizeof(RSRCICOGROUPENTRY) == 12+2 && sizeof(FILEICOGROUPENTRY) == 12+4);
   assert(sizeof(RSRCICOGROUPENTRY) == sizeof(RSRCCURGROUPENTRY));
 
@@ -767,6 +857,7 @@ bool CResourceEditor::AddExtraIconFromFile(const WINWCHAR* Type, WINWCHAR* Name,
     BYTE *pImg = (BYTE*) (((char*) Data) + FIX_ENDIAN_INT32(pSrcFGE[i].Offset)), *pCursor = 0;
 
     if (isCursor) { // We must prepend the hotspot to the image data and change the group entry
+      assert(!EditorSupportsCursorPng());
       GENERICIMAGEINFO info;
       if (/*!IsPNGFile(pImg, imgSize, &info) &&*/ !GetDIBHeaderInfo(pImg, imgSize, info)) // Are PNG cursor images allowed?
         goto fail;
@@ -800,6 +891,158 @@ bool CResourceEditor::AddExtraIconFromFile(const WINWCHAR* Type, WINWCHAR* Name,
   return !failed;
 }
 
+template<class C, class P> static bool Contains(C&Map, P*p) {
+  return p && (size_t) Map.base <= (size_t) p && (size_t) Map.base + Map.size > (size_t) p;
+}
+
+void CResourceEditor::FreeExternal(EXTERNAL&X) {
+  assert(sizeof(FILEVIEW) <= sizeof(X.Map));
+  if (X.Data) {
+    FILEVIEW &map = *(FILEVIEW*) &X.Map;
+    if (!Contains(map, X.Data)) free(X.Data);
+    close_file_view(map);
+    free(X.FreeThis); // ParseResProtocolAlloc
+  }
+}
+
+// Maps a file into memory and locates a resource inside it, a manipulated from a resource inside it or the file itself. Free with FreeExternal.
+const TCHAR* CResourceEditor::MapExternal(const TCHAR*File, TYPEMANIPULATION Manip, EXTERNAL&X) {
+  TCHAR *resproto = ParseResProtocolAlloc(File, X.RT, X.RN, X.RL);
+  if (resproto) {
+    File = resproto;
+  }
+
+  FILEVIEW &map = *(FILEVIEW*) &X.Map;
+  size_t datasize;
+  char *filedata = create_file_view_readonly(File, map), *data = 0, *dataalloc = 0;
+  if (filedata) {
+    if (resproto) {
+      signed char exetype = GetExeType(filedata, map.size);
+      if (exetype == 'P') {
+        CResourceEditor re(filedata, (int) map.size);
+        DWORD ofs, siz = 0, firstname = X.RN == (TCHAR*) FIRSTRESDIRSTRADDRESS;
+        CResourceDirectoryEntry*pNRDE, *pLRDE;
+        CResourceDataEntry*pRE = re.FindResourceT(X.RT, X.RN, X.RL, 0, &pNRDE, &pLRDE);
+        if (pRE) {
+          const WINWCHAR *wrn = pNRDE->GetNameOrId();
+          if (firstname) X.RN = IS_INTRESOURCE(wrn) ? (TCHAR*)(size_t) wrn : _T("");
+          X.RL = pLRDE->GetId();
+          ofs = pRE->GetOffset(), siz = pRE->GetSize();
+        }
+        if (siz && IsIcoCurGroupType(X.RT) && (Manip == TM_AUTO || (Manip & TM_ICON))) { // Must create a fake .ico file
+          data = dataalloc = (char*) re.ExtractIcoCur(*pRE, X.RL, datasize), siz = 0;
+        }
+        if (siz && siz != DWORD(-1)) {
+          data = filedata + ofs, datasize = siz; // Raw resource data
+        }
+      }
+    }
+    else {
+      data = filedata, datasize = map.size; // Just a normal file
+    }
+  }
+  X.Data = (BYTE*) data, X.cbData = datasize;
+  X.FreeThis = resproto;
+  if (data) return File;
+  FreeExternal(X);
+  return 0;
+}
+
+bool CResourceEditor::UpdateResourceFromExternalT(const TCHAR* Type, WORD Name, LANGID Lang, const TCHAR*File, TYPEMANIPULATION Manip) {
+  EXTERNAL x;
+  size_t &datasize = x.cbData;
+  bool success = false;
+  const TCHAR *parsedpath = MapExternal(File, Manip, x);
+  if (parsedpath) {
+    if ((DWORD) datasize == datasize) {
+      success = this->UpdateResource(Type, Name, Lang, (BYTE*) x.Data, (DWORD) datasize, Manip);
+    }
+    FreeExternal(x);
+  }
+  return success;
+}
+
+CResourceDataEntry* CResourceEditor::FindIcoCurDataEntry(WORD Type, WORD Id, LANGID PrefLang) const {
+  CResourceDataEntry*pRDE = FindResource(MAKEINTRESOURCEWINW(Type), MAKEINTRESOURCEWINW(Id), PrefLang);
+  return pRDE ? pRDE : FindResource(MAKEINTRESOURCEWINW(Type), MAKEINTRESOURCEWINW(Id), ANYLANGID);
+}
+
+BYTE* CResourceEditor::ExtractIcoCurW(const WINWCHAR* szType, const WINWCHAR* szName, LANGID wLanguage, size_t&cbData) const {
+  CResourceDirectoryEntry*pLangDir = FindResourceLanguageDirEntryW(szType, szName, wLanguage);
+  if (!pLangDir)
+    return 0;
+  CResourceDataEntry*pRDE = pLangDir->GetDataEntry();
+  return ExtractIcoCur(*pRDE, pLangDir->GetId(), cbData); // Uses the "real" LANGID
+}
+
+BYTE* CResourceEditor::ExtractIcoCur(const CResourceDataEntry&rde, LANGID ChildLang, size_t&cbData) const {
+  const CResourceDataEntry*pRDE = &rde;
+  BYTE*pSH = pRDE->GetData(), cbRGE = 14, cbFGE = 16, *pResData;
+  DWORD i, cbRes, failed = false;
+  if (pRDE->GetSize() < 6) // Must at least have a ICO file header
+    return 0;
+  WORD imgResType, count, *pFirstRGE = (WORD*) GetFirstICOCURGroupEntry(pSH, &imgResType, &count), *pRGE;
+  if (!pFirstRGE)
+    return 0;
+
+  WORD *pDH = 0, isCursor = imgResType == (size_t) RT_CURSOR;
+  DWORD imgsOfs = 6 + (count * cbFGE), cbTot = imgsOfs, cbImages = 0, grpsOfs = 6;
+
+  // Get the size of all images
+  for (i = 0, pRGE = pFirstRGE; i < count; ++i, pRGE += cbRGE / sizeof(*pRGE)) {
+    pRDE = FindIcoCurDataEntry(imgResType, ((RSRCICOGROUPENTRY*)pRGE)->Id, ChildLang);
+    if (pRDE && pRDE->GetData()) cbImages += FIX_ENDIAN_INT32(((FILEICOGROUPENTRY*)pRGE)->Size); else count = 0;
+  }
+  // Build the .ICO file
+  GENERICIMAGEINFO ii;
+  if (count && (pDH = (WORD*) malloc(cbTot += cbImages))) {
+    pDH[0] = 0x0000, pDH[1] = FIX_ENDIAN_INT16(isCursor ? 2 : 1), pDH[2] = FIX_ENDIAN_INT16(count);
+    for (i = 0, pRGE = pFirstRGE; i < count; ++i, pRGE += cbRGE / sizeof(*pRGE)) {
+      pRDE = FindIcoCurDataEntry(imgResType, ((RSRCICOGROUPENTRY*)pRGE)->Id, ChildLang);
+      pResData = pRDE->GetData(), cbRes = pRDE->GetSize();
+      FILEICOGROUPENTRY*pFGE = (FILEICOGROUPENTRY*) ((char*)pDH + grpsOfs);
+      memcpy(pFGE, pRGE, cbRGE), pFGE->Offset = FIX_ENDIAN_INT32(imgsOfs); // Initialize ICO group entry
+      DWORD cbImgFromGrp = FIX_ENDIAN_INT32(pFGE->Size), cbImg = cbImgFromGrp;
+      if (isCursor) {
+        pFGE->Width = (BYTE) FIX_ENDIAN_INT16(pRGE[0]), pFGE->Height = (BYTE) FIX_ENDIAN_INT16(pRGE[1]);
+        if (cbRes >= 4+12) {
+          assert(!EditorSupportsCursorPng());
+          pFGE->Planes = ((WORD*)pResData)[0], pFGE->BPP = ((WORD*)pResData)[0], cbImg -= 4; // Hotspot
+          DWORD cbBMH = GetDIBHeaderInfo(pResData += 4, cbRes - 4, ii), cd = ii.BPP * ii.Planes;
+          pFGE->Palette = cbBMH && cd < 8 ? (BYTE)(1 << cd) : 0; // devblogs.microsoft.com/oldnewthing/20101018-00/?p=12513 says only for depths < 8!
+          pFGE->Reserved = 0;
+        }
+        else
+          ++failed;
+      }
+      if (cbImg <= cbRes) {
+        memcpy((char*)pDH + imgsOfs, pResData, cbImg);
+        pFGE->Size = FIX_ENDIAN_INT32(cbImg);
+        imgsOfs += cbImg, grpsOfs += cbFGE;
+      }
+      else
+        ++failed;
+    }
+  }
+  cbData = cbTot;
+  if (!count || failed)
+    free(pDH), pDH = 0;
+  return (BYTE*) pDH;
+}
+
+BYTE* CResourceEditor::ExtractIcoCurT(const TCHAR* szType, WORD szName, LANGID wLanguage, size_t&cbData) const {
+  assert(!EditorSupportsStringNames() && sizeof(szName));
+#if defined(_WIN32) && defined(_UNICODE)
+  return ExtractIcoCurW((WINWCHAR*)szType, MAKEINTRESOURCEWINW(szName), wLanguage, cbData);
+#else
+  WINWCHAR* szwType = ResStringToUnicode(szType);
+  BYTE* result = ExtractIcoCurW(szwType, MAKEINTRESOURCEWINW(szName), wLanguage, cbData);
+  FreeUnicodeResString(szwType);
+  return result;
+#endif
+}
+
+
 //////////////////////////////////////////////////////////////////////
 // Private Methods
 //////////////////////////////////////////////////////////////////////
@@ -1269,7 +1512,7 @@ CResourceDataEntry::~CResourceDataEntry() {
 
 // To save memory this function doesn't give you a copy of the data
 // Don't mess with the data returned from this function!
-BYTE* CResourceDataEntry::GetData() {
+BYTE* CResourceDataEntry::GetData() const {
   return m_pbData;
 }
 
@@ -1286,14 +1529,14 @@ void CResourceDataEntry::SetData(BYTE* pbData, DWORD dwSize, DWORD dwCodePage) {
   m_dwOffset = DWORD(-1); // unset
 }
 
-DWORD CResourceDataEntry::GetSize() {
+DWORD CResourceDataEntry::GetSize() const {
   return m_dwSize;
 }
 
-DWORD CResourceDataEntry::GetCodePage() {
+DWORD CResourceDataEntry::GetCodePage() const {
   return m_dwCodePage;
 }
 
-DWORD CResourceDataEntry::GetOffset() {
+DWORD CResourceDataEntry::GetOffset() const {
   return m_dwOffset;
 }
