@@ -3,7 +3,7 @@
  * 
  * This file is a part of NSIS.
  * 
- * Copyright (C) 1999-2020 Nullsoft and Contributors
+ * Copyright (C) 1999-2021 Nullsoft and Contributors
  * 
  * Licensed under the zlib/libpng license (the "License");
  * you may not use this file except in compliance with the License.
@@ -71,6 +71,7 @@ typedef enum {
   //DE_PP_VERBOSE_BAD_LEVEL = 4000?,
   //DW_STALE_TEMP = 5020?, TODO: There is currently no way to disable this
   DW_PACKHDR_RETNONZERO = 5021,
+  DW_BADFORMAT_EXTERNAL_FILE = 5040,
   DW_UNSUPP_STORE_FILE_ATT = 5050,
   //DW_CMDLINE_UNSUPP = 5200?,
   //DW_CMDLINE_UNSUPP_WIN = 5201?,
@@ -252,6 +253,7 @@ class CEXEBuild {
     unsigned int get_header_size() const { return (unsigned int)sizeof(header) + (is_target_64bit() ? (4 * BLOCKS_NUM) : 0); }
 
     void set_default_output_filename(const tstring& filename);
+    const TCHAR* get_output_filename() const { return build_output_filename; }
 
     // process a script (you can process as many scripts as you want,
     // it is as if they are concatenated)
@@ -287,7 +289,7 @@ class CEXEBuild {
     #else
     typedef bool (*HOSTAPIREQUESTDATAPROC)(void*cookie);
     #endif
-    bool hostapi_request_data(MakensisAPI::datatransfer_e operation, UINT minver, HOSTAPIREQUESTDATAPROC proc, void*cookie, const void* input, UINT inputsize) const;
+    bool hostapi_request_data(MakensisAPI::datatransfer_e operation, UINT minver, HOSTAPIREQUESTDATAPROC proc, void*cookie, const void* input, size_t inputsize) const;
     bool prompt_for_output_path(TCHAR*path, UINT pathcap) const;
 
   private:
@@ -388,7 +390,7 @@ class CEXEBuild {
     int pp_define(LineParser&line);
     int pp_undef(LineParser&line);
     int pp_packhdr(LineParser&line);
-    int pp_finalize(LineParser&line);
+    int pp_finalize(int which_token, LineParser&line);
     int pp_execute(int which_token, LineParser&line);
     int pp_addincludedir(LineParser&line);
     int pp_include(LineParser&line);
@@ -604,7 +606,7 @@ class CEXEBuild {
       TCHAR cmd[1];
       void delete_all();
       static postbuild_cmd* make(const TCHAR *cmdstr, int cmpop, int cmpval);
-    } *postbuild_cmds;
+    } *postbuild_cmds, *postubuild_cmds;
     int run_postbuild_cmds(const postbuild_cmd *cmds, const TCHAR *templatearg_pc1, const TCHAR* commandname);
     int check_external_exitcode(int exitcode, int op, int val);
 
