@@ -3,7 +3,7 @@
  * 
  * This file is a part of NSIS.
  * 
- * Copyright (C) 1999-2020 Nullsoft and Contributors
+ * Copyright (C) 1999-2021 Nullsoft and Contributors
  * 
  * Licensed under the zlib/libpng license (the "License");
  * you may not use this file except in compliance with the License.
@@ -21,9 +21,6 @@
 
 #include "api.h"
 
-extern exec_flags_t g_exec_flags;
-extern exec_flags_t g_exec_flags_last_used;
-
 int NSISCALL ExecuteCodeSegment(int pos, HWND hwndProgress); // returns 0 on success
 int NSISCALL ExecuteCallbackFunction(int num); // returns 0 on success
 
