@@ -3,7 +3,7 @@
  * 
  * This file is a part of NSIS.
  * 
- * Copyright (C) 1999-2020 Nullsoft and Contributors
+ * Copyright (C) 1999-2021 Nullsoft and Contributors
  * 
  * Licensed under the zlib/libpng license (the "License");
  * you may not use this file except in compliance with the License.
@@ -130,7 +130,7 @@ enum
 #endif
 
 #ifdef NSIS_SUPPORT_GETDLLVERSION
-  EW_GETDLLVERSION,     // GetDLLVersion: 3 [file highout lowout]
+  EW_GETDLLVERSION,     // GetDLLVersion: 4 [file highout lowout fixedoffset]
 #endif
 
 #ifdef NSIS_SUPPORT_ACTIVEXREG
@@ -568,6 +568,18 @@ typedef struct {
 #endif
 
 
+#define GETOSINFO_KNOWNFOLDER 0
+#define GETOSINFO_READMEMORY 1
+typedef struct {
+  UINT32 WVBuild;
+  BYTE WVProd; // W9x: 0, WNT:AnyServer: & 2, WNT:DC: & 7 == 3
+  BYTE WVSP;
+  BYTE WVMin;
+  BYTE WVMaj;
+} osinfo;
+#define ABI_OSINFOOFFSET ( sizeof(exec_flags_t) )
+#define ABI_OSINFOADDRESS ( 0 )
+
 // special escape characters used in strings: (we use control codes in order to minimize conflicts with normal characters)
 #define NS_LANG_CODE  _T('\x01')    // for a langstring
 #define NS_SHELL_CODE _T('\x02')    // for a shell folder path
