@@ -3,7 +3,7 @@
  * 
  * This file is a part of NSIS.
  * 
- * Copyright (C) 1999-2020 Nullsoft and Contributors
+ * Copyright (C) 1999-2021 Nullsoft and Contributors
  * 
  * Licensed under the zlib/libpng license (the "License");
  * you may not use this file except in compliance with the License.
@@ -62,9 +62,10 @@ typedef struct
 
 typedef std::vector<Icon> IconGroup;
 
+IconGroup load_icon(const TCHAR* filename);
 IconGroup load_icon_file(const TCHAR* filename);
 IconGroup load_icon_res(CResourceEditor* re, WORD id);
-void free_loaded_icon(IconGroup icon);
+void free_loaded_icon(IconGroup&icon);
 
 void set_main_icon(CResourceEditor* re, WORD wIconId, IconGroup icon1, IconGroup icon2);
 
